package com.agentfactory.afapl.compiler;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.agentfactory.afapl.interpreter.CommitmentRule;
import com.agentfactory.afapl.interpreter.Plan;
import com.agentfactory.afapl.interpreter.Utilities;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.AND;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.IMPLIES;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.plans.BlockStep;
import com.agentfactory.clf.plans.IPlanStep;

public class CodeGeneratorVisitor implements AFAPLParserVisitor {
	private static int counter = 0;
	public Map<Predicate, String> actions = new HashMap<Predicate, String>();
	public Map<Predicate, String> sensors = new HashMap<Predicate, String>();
	public Map<Predicate, String> modules = new HashMap<Predicate, String>();
	public List<CommitmentRule> rules = new LinkedList<CommitmentRule>();
	public List<Predicate> beliefs = new LinkedList<Predicate>();
	public List<Plan> plans = new LinkedList<Plan>();
	public List<IMPLIES> inferences = new LinkedList<IMPLIES>();

	@SuppressWarnings("unchecked")
	@Override
	public Object visit(SimpleNode node, Object data) {
		PredicateBuilderVisitor visitor = new PredicateBuilderVisitor();
		if (node.toString().equals("SensorExpression")) {
			String predicate = (String) visitor.visit((SimpleNode) node.jjtGetChild(0), null);
			String cname = (String) ((SimpleNode) node.jjtGetChild(1)).jjtGetValue();
			if (cname != null) {
				sensors.put((Predicate) Utilities.factory.createFormula(predicate), cname);
			}
		} else if (node.toString().equals("InferenceExpression")) {
			AND and = (AND) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null);
			String rhs = (String) new PredicateBuilderVisitor().visit((SimpleNode) node.jjtGetChild(1), null);
			inferences.add(new IMPLIES(and, CoreUtilities.factory.createFormula(rhs)));
		} else if (node.toString().equals("ActionExpression")) {
			String predicate = (String) visitor.visit((SimpleNode) node.jjtGetChild(0), null);
			String cname = (String) ((SimpleNode) node.jjtGetChild(1)).jjtGetValue();
			if (cname != null) {
				actions.put((Predicate) Utilities.factory.createFormula(predicate), cname);
			}
		} else if (node.toString().equals("ModuleExpression")) {
			String predicate = (String) visitor.visit((SimpleNode) node.jjtGetChild(0), null);
			String cname = (String) ((SimpleNode) node.jjtGetChild(1)).jjtGetValue();
			if (cname != null) {
				modules.put((Predicate) Utilities.factory.createFormula(predicate), cname);
			}
		} else if (node.toString().equals("PlanExpression")) {
			String name = (String) new PredicateBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null);
			int i = 1;
			IFormula pre = null;
			IFormula post = null;
			BlockStep plan = null;
			while (i < node.jjtGetNumChildren()) {
				SimpleNode n = (SimpleNode) node.jjtGetChild(i++);
				if (n.jjtGetChild(0).toString().equals("PreCondition")) {
					pre = (IFormula) new SentenceBuilderVisitor().visit((SimpleNode) n.jjtGetChild(0).jjtGetChild(0), null);
				} else if (n.jjtGetChild(0).toString().equals("PostCondition")) {
					post = (IFormula) new SentenceBuilderVisitor().visit((SimpleNode) n.jjtGetChild(0).jjtGetChild(0), null);
				} else if (n.jjtGetChild(0).toString().equals("Block")) {
					plan = (BlockStep) new PlanBuilderVisitor().visit((SimpleNode) n.jjtGetChild(0), null);
				}
			}
			plans.add(new Plan((Predicate) Utilities.factory.createFormula(name), pre, post, plan));
		} else if (node.toString().equals("RuleExpression")) {
			int i = 0;
			Predicate label = null;
			if (node.jjtGetNumChildren() == 3) {
				label = (Predicate) Utilities.factory.createFormula((String) new PredicateBuilderVisitor().visit((SimpleNode) node.jjtGetChild(i++), null));
			} else {
				label = (Predicate) Utilities.factory.createFormula("rule" + (counter++));
			}
			IFormula condition = (IFormula) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(i++), null);
			List<IPlanStep> plan = (List<IPlanStep>) new PlanBuilderVisitor().visit((SimpleNode) node.jjtGetChild(i++), null);
			rules.add(new CommitmentRule( label, condition, plan ));
		} else if (node.toString().equals("Predicate")) {
			String predicate = (String) new PredicateBuilderVisitor().visit(node, null);
			beliefs.add((Predicate) Utilities.factory.createFormula(predicate));
		} else if (node.toString().equals("IncludeExpression")) {
			// IGNORE: This should never be called
		} else {
			for (int i=0; i<node.jjtGetNumChildren(); i++) {
				visit((SimpleNode) node.jjtGetChild(i), data);
			}
		}
		return null;
	}

}
