package com.agentfactory.afapl.interpreter;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.plans.IPlanStep;
import com.agentfactory.clf.plans.PlanStack;
import com.agentfactory.clf.reasoner.Bindings;

public class AFAPLAchieveStep implements IPlanStep {
	private static final int NEW							= 0;
	private static final int ACTIVE							= 1;
	
	protected IFormula step;
	protected int state;
	protected Goal goal;
	protected GoalHandler handler;
	
	public AFAPLAchieveStep(IFormula step) {
		this.step = step;
		state = NEW;
	}
	
	@Override
	public IPlanStep apply(Bindings set) {
		return new AFAPLAchieveStep(step.apply(set));
	}
	
	public IFormula formula() {
		return step;
	}

	public String toString() {
		return toString("");
	}
	
	public String toString(String tabs) {
		return tabs+Utilities.presenter.toString(step);
	}

	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		AbstractAFAPLAgent agt = (AbstractAFAPLAgent) agent;
		
		switch (state) {
		case NEW:
			planStack.back();
			goal = (Goal) step.apply(planStack.bindings());
			handler = agt.addGoal(goal);
			handler.setSource((Commitment) planStack);
			state = ACTIVE;
			break;
		case ACTIVE:
			// Wait for the agent to process the goal:
			// - if it was successful, continue
			// - if it failed, fail the plan
			if (handler.state() == GoalHandler.SUCCEEDED) {
				goal = null;
				handler = null;
				return;
			} else if (handler.state() == GoalHandler.FAILED) {
				goal = null;
				handler = null;
				planStack.fail();
			} else {
				planStack.back();
			}
			break;
		}
	}
	
}
