package com.agentfactory.afapl.interpreter;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.lang.Unifier;
import com.agentfactory.clf.plans.BasicStep;
import com.agentfactory.clf.plans.PlanStack;
import com.agentfactory.clf.reasoner.Bindings;
import com.agentfactory.clf.reasoner.QueryResult;

public class AFAPLBasicStep extends BasicStep {
	public AFAPLBasicStep(IFormula step) {
		super(step);
	}
	
	@Override
	public void handle(AbstractLanguageAgent ala, PlanStack planStack) {
		Commitment commitment = (Commitment) planStack;
		if (Predicate.class.isInstance(step)) {
			Predicate activity = (Predicate) step.apply(commitment.bindings());
			AbstractAFAPLAgent agent = (AbstractAFAPLAgent) ala;
			String sig = Utilities.createSignature(activity);
			for (Plan plan : agent.getPlans()) {
				String sig2 = Utilities.createSignature(plan.identifier);
				if (sig.equals(sig2)) {
					Bindings b = Unifier.unifyFormulae(activity, plan.identifier);
					Plan p = plan.apply(b);
					
					QueryResult result = agent.queryAll(p.pre);
					if (result.found) {
						if (result.bindings.isEmpty()) {
							commitment.addPlan(p.plan.plan());
						} else {
							b = result.bindings.get(0);
							commitment.addPlan(p.apply(b).plan.plan());
						}
					}
					return;
				}
			}
			super.handle(ala, planStack);
		}
	}
	
	public BasicStep apply(Bindings set) {
		return new AFAPLBasicStep(step.apply(set));
	}
}
