package com.agentfactory.afapl.interpreter;

import com.agentfactory.afapl.compiler.AFAPLParser;
import com.agentfactory.afapl.compiler.CodeGeneratorVisitor;
import com.agentfactory.afapl.compiler.IncludesExpanderStream;
import com.agentfactory.afapl.compiler.SimpleNode;
import com.agentfactory.clf.interpreter.Action;
import com.agentfactory.clf.interpreter.Module;
import com.agentfactory.clf.interpreter.Sensor;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.platform.core.AgentCreationException;

public class ScriptBasedAFAPLAgent extends AbstractAFAPLAgent {

	public ScriptBasedAFAPLAgent(String name) {
		super(name);
		
		this.initialise("state(initialized)");
	}

	public void construct(String design) throws AgentCreationException {
		CodeGeneratorVisitor cgv = new CodeGeneratorVisitor();
		try {
			AFAPLParser parser = new AFAPLParser(new IncludesExpanderStream(design));
			SimpleNode n = parser.Start();
			cgv.visit(n, null);
			if (!parser.errors.isEmpty()) {
				throw new RuntimeException("ERROR: AFAPL program does not compile: " + design);
			}
			
			for (Predicate belief: cgv.beliefs) {
				addBelief(belief);
			}
			
			for (Predicate id: cgv.actions.keySet()) {
				Action action = (Action) Class.forName(cgv.actions.get(id)).newInstance();
				action.setId(id);
				action.setAgent(this);
				actionMap.put(Utilities.createSignature(id), action);
			}
			
			for (Predicate id: cgv.sensors.keySet()) {
				Sensor sensor = (Sensor) Class.forName(cgv.sensors.get(id)).newInstance();				
				sensor.setId(id);
				sensor.setAgent(this);
				sensorMap.put(Utilities.createSignature(id), sensor);
			}
			
			for (Predicate id: cgv.modules.keySet()) {
				addModule(id, (Module) Class.forName(cgv.modules.get(id)).newInstance());
			}
			
			for (CommitmentRule rule: cgv.rules) {
				addCommitmentRule(rule);
			}
			
			this.plans = cgv.plans;
		} catch (Throwable e) {
			e.printStackTrace();
			throw new AgentCreationException(e.getMessage());
		}
	}
}
