/*
 * ProgramNode.java
 *
 * Created on 13 June 2006, 14:54
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.parser.node;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.agentfactory.afapl2.compiler.parser.LexicalAnalyser;
import com.agentfactory.afapl2.compiler.parser.LexicalAnalyserException;
import com.agentfactory.afapl2.compiler.parser.ParseTreeException;
import com.agentfactory.afapl2.compiler.parser.ParserHelper;
import com.agentfactory.afapl2.compiler.parser.token.Token;

/**
 * ACTION construct syntax (no initialization):
 * 
 * ACTION <PredicateNode> {
 *     PRECONDITION <node>;
 *     POSTCONDITION <node>;
 *     CLASS <node>;
 * }
 * 
 * ACTION construct syntax (plus initialization):
 * 
 * ACTION <fosNode> {
 *     PRECONDITION <node>;
 *     POSTCONDITION <node>;
 *     CLASS <node> {
 *         key=value;
 *         key=value;
 *     }
 * }
 *
 * @author Administrator
 */
public class ActionNode extends Node {
    private ActivityNode identifier;
    private Node preCondition;
    private Node postCondition;
    private FOSNode cardinality;
    private ClassNode actuator;
    private Map<String, String> configuration;
    private List<String> ontologies;
    private List<Node> postConditions;
    
    /** Creates a new instance of ProgramNode */
    public ActionNode() {
        configuration = new HashMap<String, String>();
        ontologies = new LinkedList<String>();
        postConditions = new LinkedList<Node>();
    }
    
    @Override
	public void construct(LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        // Step 1: Extract identifier
        Token token = analyser.getNextToken();
        identifier = ActivityNode.create(token, analyser);
        
        // Step 2: Check that we have an open brace '{'
        ParserHelper.punctuationCheck("{", "Syntax Error in ACTION " + identifier, "expected { but got", analyser);
        
        // Step 3: Extract precondition, postcondition, and class
        token = analyser.getNextToken();
        while (!token.getText().equals("}")) {
            if (token.getText().equals("PRECONDITION")) {
                preCondition = ParserHelper.checkForLogicOperators(";", Node.create(analyser.getNextToken(), analyser), token, analyser);
                ParserHelper.punctuationCheck(";", "Syntax Error in PRECONDITION parameter of ACTION " + identifier, "expected ; but got", analyser);
            } else if (token.getText().equals("EFFECT") || token.getText().equals("POSTCONDITION")) {
                // TODO (Rem Collier): POSTCONDITION is maintained for backwards compatibility.
                // need to refactor this at a later point when new approach is stabalised.
                postCondition = ParserHelper.checkForLogicOperators(";", Node.create(analyser.getNextToken(), analyser), token, analyser);
                ParserHelper.punctuationCheck(";", "Syntax Error in EFFECT parameter of ACTION " + identifier, "expected ; but got", analyser);
                postConditions.add(postCondition);
            } else if (token.getText().equals("CLASS")) {
                actuator = ClassNode.createClassNode(analyser.getNextToken(), analyser);
                token = analyser.getNextToken();
                if (token.getText().equals("{")) {
                    parseConfiguration(analyser);
                } else {
                    analyser.back(token);
                    ParserHelper.punctuationCheck(";", "Syntax Error in CLASS parameter of ACTION " + identifier, "expected ; but got", analyser);
                }
            } else if (token.getText().equals("USES")) {
                ontologies.add(analyser.getNextToken().getText());
                ParserHelper.punctuationCheck(";", "Syntax Error in USES parameter of ACTION " + identifier, "expected ; but got", analyser);
            } else if (token.getText().equals("CARDINALITY")) {
                cardinality = FOSNode.createFOS(analyser.getNextToken(), analyser);
                ParserHelper.punctuationCheck(";", "Syntax Error in CARDINALITY parameter of ACTION " + identifier, "expected ; but got", analyser);
            } else {
                throw new ParseTreeException("Syntax Error in ACTION " + identifier, "expected PRECONDITION, EFFECT, or CLASS but got", token);
            }
            
            token = analyser.getNextToken();
        }        
        
        if (preCondition == null) {
        	  preCondition = new BeliefNode(new FOSNode("true"));
        } 
        if (postCondition == null) {
            postCondition = new BeliefNode(new FOSNode("true"));
            postConditions.add(postCondition);
        } 
        if (actuator == null) {
            throw new LexicalAnalyserException(analyser.getSource(), 0, "ACTION " + identifier + " missing CLASS parameter");
        }
        
        if (cardinality == null) {
            Token temp = Token.createToken(";", token.getLine(), token.getReader());
            analyser.back(temp);
            token = Token.createToken("0", token.getLine(), token.getReader());
            cardinality = FOSNode.createFOS(token, analyser);
        }

        token = analyser.getNextToken();
        if (token != null) {
            analyser.back(token);
        }
        
        if ((token == null) || !token.getText().equals(";")) {
            Token temp = Token.createToken(";", -1, null);
            analyser.back(temp);
        }
    }
    
    private void parseConfiguration(LexicalAnalyser analyser) throws ParseTreeException,LexicalAnalyserException {
        String key = null;
        String value = null;
        Token token = analyser.getNextToken();
        while (!token.getText().equals("}")) {
            key = token.getText();
            ParserHelper.punctuationCheck("=", "Syntax Error in CLASS parameter configuration of ACTION " + identifier, "expected = but got", analyser);
            value = analyser.getNextToken().getText();
            ParserHelper.punctuationCheck(";", "Syntax Error in CLASS parameter configuration of ACTION " + identifier, "expected ; but got", analyser);
            token = analyser.getNextToken();
            configuration.put(key, value);
        }
    }
    
    @Override
	public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("ACTION " + identifier + " { PRECONDITION " + preCondition + ";");
        Iterator it = postConditions.iterator();
        while (it.hasNext()) {
            buf.append("EFFECT " + it.next() + ";");
        }
        buf.append(" CARDINALITY " + cardinality + "; CLASS " + actuator + ";}");
        return buf.toString();
    }
    
    public ClassNode getActionClass() {
        return actuator;
    }
    
    public ActivityNode getIdentifier() {
        return identifier;
    }
    
    public Node getPreCondition() {
        return preCondition;
    }
    
    public Node getPostCondition() {
        return postCondition;
    }
    
    public FOSNode getCardinality() {
        return cardinality;
    }
    
    public Map getConfiguration() {
        return configuration;
    }
    
    public List<String> getOntologies() {
        return ontologies;
    }
    
    public List<Node> getPostConditions() {
        return postConditions;
    }
}
