/*
 * ProgramNode.java
 *
 * Created on 13 June 2006, 14:54
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.parser.node;

import java.util.ArrayList;
import java.util.List;

import com.agentfactory.afapl2.compiler.parser.LexicalAnalyser;
import com.agentfactory.afapl2.compiler.parser.LexicalAnalyserException;
import com.agentfactory.afapl2.compiler.parser.ParseTreeException;
import com.agentfactory.afapl2.compiler.parser.ParserHelper;
import com.agentfactory.afapl2.compiler.parser.token.PunctuationToken;
import com.agentfactory.afapl2.compiler.parser.token.Token;
/**
 *
 * @author Administrator
 */
public class EnumNode extends Node {
    
    public static EnumNode create(Token token, LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        if (token instanceof PunctuationToken) {
            throw new ParseTreeException("Syntax Error in Predicate Definition", "expected predicate but got", token);
        }
        
        EnumNode node = new EnumNode(token.getText());
        node.construct(analyser);
        node.setInitialToken(token);
        return node;
    }
    
    private String identifier;
    private List<String> constants;
    
    /** Creates a new instance of ProgramNode */
    public EnumNode(String identifier) {
        this.identifier = identifier;
        this.constants = new ArrayList<String>();
    }

    @Override
	public void construct(LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        // Syntax: <identifier> {<const>[,<const>]}

        // Step 1: Check if we have an open bracket "{"
        Token token = analyser.getNextToken();
        if (!token.getText().equals("{")) {
            throw new ParseTreeException("Syntax Error in ENUM Definition for: '" + identifier + "'", "expected { but found", token);
        }
        
        // Step 2: Get the first variable definition
        addConstant(analyser);
        
        token = analyser.getNextToken();
        while (token.getText().equals(",")) {
            addConstant(analyser);
            token = analyser.getNextToken();
        }
        analyser.back(token);
        ParserHelper.punctuationCheck("}", "Syntax Error in ENUM definition for: " + identifier, "expected ) but got", analyser);
    }

    @Override
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("ENUM " + identifier);
        buf.append(" {");
        boolean first = true;
        int i=0;
        while (i<constants.size()) {
            if (first) {
                first = false;
            } else {
                buf.append(",");
            }
            buf.append(constants.get(i++));
        }
        buf.append("}");
        return buf.toString();
    }
    
    private void addConstant(LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        Token token = analyser.getNextToken();
        if (token.getText().startsWith("?")) {
            throw new ParseTreeException("Syntax Error in ENUM Definition for: " + identifier, "variables not permitted: ", token);
        } else {
            constants.add(token.getText());
        }
    }
    
    
    public String getIdentifier() {
        return identifier;
    }
    
    public List<String> getConstants() {
        return constants;
    }
}
