/*
 * ProgramNode.java
 *
 * Created on 13 June 2006, 14:54
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.parser.node;

import java.util.ArrayList;
import java.util.List;

import com.agentfactory.afapl2.compiler.parser.LexicalAnalyser;
import com.agentfactory.afapl2.compiler.parser.LexicalAnalyserException;
import com.agentfactory.afapl2.compiler.parser.ParseTreeException;
import com.agentfactory.afapl2.compiler.parser.token.PunctuationToken;
import com.agentfactory.afapl2.compiler.parser.token.Token;
/**
 *
 * @author Administrator
 */
public class FOSNode extends Node {
    
    public static FOSNode createFOS(Token token, LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        if (token instanceof PunctuationToken) {
            throw new ParseTreeException("Syntax Error in first-order structure", "expected constant or variable but got", token);
        }
        
        FOSNode node = new FOSNode(token.getText());
        node.construct(analyser);
        node.setInitialToken(token);
        return node;
    }
    
    private String functor;
    private List<FOSNode> arguments;
    
    /** Creates a new instance of ProgramNode */
    public FOSNode(String functor) {
        this.functor = functor;
        this.arguments = new ArrayList<FOSNode>();
    }
    
    @Override
	public void construct(LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        // Syntax: <functor>[(<fosNode>[,<fosNode>]*)]

        // Step 1: Check if we have an open bracket "("
        Token token = analyser.getNextToken();
        if (functor.equals("!")) {
            functor += token.getText();
            token = analyser.getNextToken();
        }
        
        if (token instanceof PunctuationToken) {
            if (token.getText().equals(")") || token.getText().equals(",") || 
                    token.getText().equals(";") || token.getText().equals("{")) {
                // we have a constant or variable, so nothing to construct...
                // but first we have to put the token back into the analyser...
                analyser.back(token);
                return;
            } else if (!token.getText().equals("(")) {
                throw new ParseTreeException("Syntax Error in first-order structure '" + functor + "'", "expected ( or , but found", token);
            }
        } else {
            System.out.println("ERROR: " + token.getText());
            // Didn't get punctuation, so we have a serious problem
            throw new ParseTreeException("Syntax Error in first-order structure '" + functor + "'", "expected ( or , but found", token);
        }
        
        // Step 2: We have arguments, so add the first one
        token = analyser.getNextToken();
        arguments.add(FOSNode.createFOS(token, analyser));

        // Step 3: Now check how many more arguments we have...
        token = analyser.getNextToken();
        while (token.getText().equals(",")) {
            token = analyser.getNextToken();
            arguments.add(FOSNode.createFOS(token, analyser));
            token = analyser.getNextToken();
        }
        
        // Step 4: Check for the close bracket at the end...
        if (token instanceof PunctuationToken) {
            if (!token.getText().equals(")")) {
                // we expected a close bracket but didn't get one, so throw an
                // exception...
                throw new ParseTreeException("Syntax Error in first-order structure '" + functor + "'", "expected ) or , but found", token);
            }
        } else {
            // Didn't get punctuation, so we have a serious problem
            throw new ParseTreeException("Syntax Error in first-order structure '" + functor + "'", "expected ( or , but found", token);
        }
    }
    
    @Override
	public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append(functor);
        if (!arguments.isEmpty()) {
            buf.append("(");
            boolean first = true;
            int i=0;
            while (i<arguments.size()) {
                if (first) {
                    first = false;
                } else {
                    buf.append(",");
                }
                buf.append(arguments.get(i++));
            }
            buf.append(")");
        } 
        return buf.toString();
    }
    
    @Override
	public boolean equals(Object obj) {
        if (!(obj instanceof FOSNode)) {
            return false;
        }
        
        FOSNode fos = (FOSNode) obj;
        if (!this.functor.equals(fos.functor)) {
            return false;
        }
        
        if (this.arguments.size() != fos.arguments.size()) {
            return false;
        }

        return true;
    }
    
    public String getFunctor() {
        return functor;
    }
    
    public List<FOSNode> getArguments() {
        return arguments;
    }
    
    public boolean isConstant() {
        return !functor.startsWith("?") && arguments.isEmpty();
    }
    
    public boolean isVariable() {
        return functor.startsWith("?") && arguments.isEmpty();
    }
    
    public boolean isComposite() {
        return !functor.startsWith("?") && !arguments.isEmpty();
    }
    
    public boolean isTrueOrFalse() {
        return ((functor.toLowerCase().equals("true") ||
             functor.toLowerCase().equals("false")) &&
             arguments.isEmpty());
    }
}
