/*
 * ProgramNode.java
 *
 * Created on 13 June 2006, 14:54
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.parser.node;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.agentfactory.afapl2.compiler.parser.LexicalAnalyser;
import com.agentfactory.afapl2.compiler.parser.LexicalAnalyserException;
import com.agentfactory.afapl2.compiler.parser.ParseTreeException;
import com.agentfactory.afapl2.compiler.parser.ParserHelper;
import com.agentfactory.afapl2.compiler.parser.token.Token;

/**
 * PEREPTOR construct syntax (no initialization):
 * 
 * ACTION <fosNode> {
 *     CLASS <node>;
 * }
 * 
 * ACTION construct syntax (plus initialization):
 * 
 * ACTION <fosNode> {
 *     CLASS <node> {
 *         key=value;
 *         key=value;
 *     }
 * }
 *
 * @author Administrator
 */
public class PerceptorNode extends Node {
    private FOSNode identifier;
    private ClassNode node;
    private Map<String,String> configuration;
    private List<String> ontologies;
    
    /** Creates a new instance of ProgramNode */
    public PerceptorNode() {
        configuration = new HashMap<String,String>();
        ontologies = new ArrayList<String>();
    }
    
    @Override
	public void construct(LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        // Step 1: Extract identifier
        Token token = analyser.getNextToken();
        identifier = FOSNode.createFOS(token, analyser);
        
        // Step 2: Check that we have an open brace '{'
        ParserHelper.punctuationCheck("{", "Syntax Error in PERCEPTOR " + identifier, "expected { but got", analyser);
        
        // Step 3: Extract precondition, postcondition, and class
        token = analyser.getNextToken();
        while (!token.getText().equals("}")) {
            if (token.getText().equals("CLASS")) {
                node = ClassNode.createClassNode(analyser.getNextToken(), analyser);
                token = analyser.getNextToken();
                if (token.getText().equals("{")) {
                    parseConfiguration(analyser);
                } else {
                    analyser.back(token);
                    ParserHelper.punctuationCheck(";", "Syntax Error in CLASS parameter of PERCEPTOR " + identifier, "expected ; but got", analyser);
                }
            } else if (token.getText().equals("USES")) {
                ontologies.add(analyser.getNextToken().getText());
                ParserHelper.punctuationCheck(";", "Syntax Error in USES parameter of PERCEPTOR " + identifier, "expected ; but got", analyser);
            } else {
                throw new ParseTreeException("Syntax Error in PEREPTOR " + identifier, "expected CLASS but got", token);
            }
            
            token = analyser.getNextToken();
        }        
        
        token = analyser.getNextToken();
        if (token != null) {
            analyser.back(token);
        }
        
        if ((token == null) || !token.getText().equals(";")) {
            Token temp = Token.createToken(";", -1, null);
            analyser.back(temp);
        }
    }
    
    private void parseConfiguration(LexicalAnalyser analyser) throws ParseTreeException,LexicalAnalyserException {
        String key = null;
        String value = null;
        Token token = analyser.getNextToken();
        while (!token.getText().equals("}")) {
            key = token.getText();
            ParserHelper.punctuationCheck("=", "Syntax Error in CLASS parameter configuration of PERCEPTOR " + identifier, "expected = but got", analyser);
            value = analyser.getNextToken().getText();
            ParserHelper.punctuationCheck(";", "Syntax Error in CLASS parameter configuration of PEREPTOR " + identifier, "expected ; but got", analyser);
            token = analyser.getNextToken();
            configuration.put(key, value);
        }
    }
    
    
    @Override
	public String toString() {
        return "PERCEPTOR " + identifier + " { CLASS " + node.toString() + "}";
    }
    
    public ClassNode getPerceptor() {
        return node;
    }
    
    public FOSNode getIdentifier() {
        return identifier;
    }
    
    public Map getConfiguration() {
        return configuration;
    }
    
    public List getOntologies() {
        return ontologies;
    }
}
