/*
 * ProgramNode.java
 *
 * Created on 13 June 2006, 14:54
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.compiler.parser.node;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.agentfactory.afapl2.compiler.parser.LexicalAnalyser;
import com.agentfactory.afapl2.compiler.parser.LexicalAnalyserException;
import com.agentfactory.afapl2.compiler.parser.ParseTreeException;
import com.agentfactory.afapl2.compiler.parser.ParserHelper;
import com.agentfactory.afapl2.compiler.parser.token.Token;

/**
 *
 * @author Administrator
 */
public class RoleNode extends Node {
    private FOSNode identifier;
    private List<TriggerNode> triggers;
    private List<ImpliesNode> rules;
    private List<Node> mentalState;
    
    /** Creates a new instance of ProgramNode */
    public RoleNode() {
        triggers = new LinkedList<TriggerNode>();
        rules = new LinkedList<ImpliesNode>();
        mentalState = new LinkedList<Node>();
    }
    
    @Override
	public void construct(LexicalAnalyser analyser) throws LexicalAnalyserException, ParseTreeException {
        // ROLE construct syntax:
        // ROLE<fosNode> {
        //     [<trigger>|<rule>]*
        // }
        
        // Step 1: Extract identifier
        Token token = analyser.getNextToken();
        identifier = FOSNode.createFOS(token, analyser);
        
        // Step 2: Check that we have an open brace '{'
        ParserHelper.punctuationCheck("{", "Syntax Error in ROLE " + identifier, "expected { but got", analyser);
        
        // Step 3: Extract precondition, postcondition, and class
        Node node = null;
        token = analyser.getNextToken();
        while (!token.getText().equals("}")) {
            node = Node.create(token, analyser);
            
            // Check if we have a BELIEF, ALWAYS, ...
            if (node instanceof ModalNode) {
                // Check for belief sentence
                node = ParserHelper.checkForLogicOperators(";", node, token, analyser);
                
                // Get the next token, to skip the reinserted ";"
                token = analyser.getNextToken();
                mentalState.add(node);
            }
            
            if (node instanceof TriggerNode) {
                triggers.add((TriggerNode) node);
            } else if (node instanceof CommitNode) {
                token = analyser.getNextToken();
                if (token.getText().equals(";")) {
                    mentalState.add(node);
                } else {
                    throw new ParseTreeException("Syntax Error in ROLE " + identifier + " near " + node, "expected ; but got ", token);
                }
            } else if (node instanceof ImpliesNode) {
                rules.add((ImpliesNode) node);
            } else {
                throw new ParseTreeException("Syntax Error in ROLE " + identifier, "expected TRIGGER or Rule but got", token);
            }
            
            token = analyser.getNextToken();
        }        
        
        token = analyser.getNextToken();
        if (token != null) {
            analyser.back(token);
        }
        
        if ((token == null) || !token.getText().equals(";")) {
            Token temp = Token.createToken(";", -1, null);
            analyser.back(temp);
        }
    }
    
    
    @Override
	public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("ROLE " + identifier +  " {");
        for (TriggerNode node:triggers) {
            buf.append(node + ";");
        }
        
        for(Node node:mentalState) {
            buf.append(node + ";");
        }
        
        for (ImpliesNode node:rules) {
            buf.append(node + ";");
        }
        
        buf.append("}");
        return buf.toString();
    }
    
    public FOSNode getIdentifier() {
        return identifier;
    }
    
    public List<TriggerNode> getTriggers() {
        return triggers;
    }
    
    public List<ImpliesNode> getRules() {
        return rules;
    }
    
    public List<Node> getMentalState() {
        return mentalState;
    }
}
