/**
 * Title:       StringReader.java
 * Copyright:   Copyright (c) 1996-2004 The Agent Factory Working Group. All rights reserved.
 * Licence:     This file is free software; you can redistribute it and/or modify
 *              it under the terms of the GNU Lesser General Public License as published by
 *              the Free Software Foundation; either version 2.1, or (at your option)
 *              any later version.
 *
 *              This file is distributed in the hope that it will be useful,
 *              but WITHOUT ANY WARRANTY; without even the implied warranty of
 *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *              GNU Lesser General Public License for more details.
 *
 *              You should have received a copy of the GNU Lesser General Public License
 *              along with Agent Factory; see the file COPYING.  If not, write to
 *              the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *              Boston, MA 02111-1307, USA.
 */
package com.agentfactory.afapl2.compiler.parser.reader;

/**
 * This class implements the Reader interface to support reading of characters
 * from strings.  It is one of a number of reader classes that may be used to
 * compile AF-APL code.
 *
 * @author  Rem Collier
 */
public class StringReader implements Reader {
    private String code;
    private int index;
    
    /**
     * Creates a new instance of StringReader
     * @param code the code to be compiled
     */
    public StringReader(String code) {
        this.code = code;
        this.index = 0;
    }
    
    /**
     * gets the next character from the reader.
     * @return the next character or '\0'
     */
    public char nextCharacter() {
        if (index == code.length()) {
            return '\0';
        }
        
        return code.charAt(index++);
    }
    
    /**
     * used to undo the nextCharacter method.
     * @param ch the character to be put back into the reader.
     */
    public void pushBack(char ch) {
        index--;
    }
    
    public void close() {
    }
    
    @Override
	public String toString() {
        int length = 10;
        if (code.length() < 10) length = code.length();
        return "StringReader open on: '" + code.substring(0, length) + "...'";
    }
}
