/*									tab:4
 * Copyright (c) 2007 University College Dublin.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL UNIVERSITY COLLEGE DUBLIN BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF 
 * UNIVERSITY COLLEGE DUBLIN HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * UNIVERSITY COLLEGE DUBLIN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND UNIVERSITY COLLEGE DUBLIN HAS NO
 * OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 *
 *
 */
package com.agentfactory.afapl2.core.actuator;

import java.util.List;

import com.agentfactory.afapl2.core.module.EISControlModule;
import com.agentfactory.logic.agent.Actuator;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.reasoner.SubstitutionSet;
import com.agentfactory.logic.reasoner.Unifier;

/**
 *
 * @author  Mauro Dragone
 */
public class AddToAuxiliaryMemory extends Actuator {

    @Override
	public boolean act(FOS action) {
        EISControlModule auxMemory = (EISControlModule) getModuleByName("auxiliaryMemory");
        if (auxMemory == null) {
            adoptBelief("BELIEF(noSuchModule(auxiliaryMemory))");
            return false;
        }

        String memoryName = action.argAt(0).toString();
        FOS fact = action.argAt(1);
        List<FOS> variables = fact.getVariables();
        if (variables.isEmpty()) {
            auxMemory.add(memoryName, fact);
            return true;
        }

        List<FOS> agentBeliefs = agent.getBeliefManager().getBeliefs();

        for (FOS bel : agentBeliefs) {
            FOS fos = bel.copy();
            if (!fos.getFunctor().equals("BELIEF")) {
                continue;
            }
            if (fos.argAt(0).matches(fact)) {
                SubstitutionSet subSet = Unifier.unifyFormulae(fos.argAt(0), fact);
                if (subSet != null) {
                    auxMemory.add(memoryName, fos.argAt(0));
                }
            }
        }




        return true;
    }
}
