package com.agentfactory.afapl2.core.actuator;

import com.agentfactory.logic.agent.Actuator;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.platform.core.AgentCreationException;
import com.agentfactory.platform.core.DuplicateAgentNameException;
import com.agentfactory.platform.core.NoSuchArchitectureException;
import com.agentfactory.service.ams.AgentManagementService;

/**
 * This actuator can be used to create new agents at run-time. To use it, the
 * agent must first be bound to the AgentManagementService platform service.
 * 
 * The corresponding action must take two parameters:
 * <ul>
 * <li>name: the name of the agent
 * <li>design: the design of the agent
 * </ul>
 * 
 * @author  remcollier
 */
public class CreateAgent extends Actuator {

    @Override
	public boolean act(FOS action) {
        String name = action.argAt(0).toString();
        String design = action.argAt(1).toString();
        
        AgentManagementService ams = (AgentManagementService) getService(AgentManagementService.NAME);
        if (ams == null) {
            adoptBelief("BELIEF(notBoundToService(" + AgentManagementService.NAME + "))");
            return false;
        }
        try {
            ams.createAgent(name, design);
        } catch (NoSuchArchitectureException ex) {
            adoptBelief("BELIEF(unknownAgentType(" + design.substring(design.lastIndexOf(".")) + "))");
            return false;
        } catch (DuplicateAgentNameException ex) {
            adoptBelief("BELIEF(agentExists(" + name + "))");
            return false;
        } catch (AgentCreationException ex) {
            adoptBelief("BELIEF(unknownAgentDesign(" + design + "))");
			return false;
        }
        
        adoptBelief("BELIEF(createdAgent(" + name + "))");
        ams.resumeAgent(name);
        return true;
    }
}
