package com.agentfactory.afapl2.core.actuator;

import java.util.logging.Level;
import java.util.logging.Logger;

import com.agentfactory.logic.agent.Actuator;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.util.Support;
import com.agentfactory.platform.core.AgentID;
import com.agentfactory.platform.impl.StringMessage;

/**
 * Actuator that sends a FIPA inform message using a HTTP MTS.  Currently only
 * allows sending of messages to a single agent.
 *
 * The action takes the form of inform(?agt, ?content)
 *
 * @author Rem Collier
 * @version 1.0
 */
public class GeneralFIPAActuator extends Actuator {
    private static final String LOGNAME = GeneralFIPAActuator.class.getName();
    
    static {
        Logger.getLogger(LOGNAME).setLevel(Level.WARNING);
    }

    @Override
	public boolean act(FOS action) {
        FOS agt = action.argAt(0);
        FOS content = action.argAt(1);

        String protocol = "none";
        String language = "AFAPL";
        if (action.numArguments() > 2) {
            protocol = action.argAt(2).toString();
        }
        if (action.numArguments() == 4) {
            language = action.argAt(3).toString();
        }
        
        //String ontology = agent.getOntologyManager().getOntologyFor(content.getFunctor());
        //if (ontology == null) {
            String ontology = "None";
        //}
        
        AgentID receiver = null;
        
        if (agt.getFunctor().equals("agentID")) {
            receiver = Support.convert(agt);
        }
        
        String performative = (String) configuration.get("performative");

        if (receiver == null) {
            adoptBelief("BELIEF(unknownAgent(" + agt + "))");            
        } else {
            StringMessage message = StringMessage.newInstance();

            // TODO: Dave: Why is this uppercase?
            message.setPerformative(performative.toUpperCase());
            
            message.setSender(agent.getAgentID());
            message.getReceivers().add(receiver);
            message.setContent(content.toString());
            message.setLanguage(language);
            message.setOntology(ontology);
            message.setProtocol(protocol);
            try {
                boolean sent = agent.send(message);
                if ( sent ) {
                    adoptBelief("BELIEF(messageSent(" + performative + "," + agt + "," + content + "))");
                } else {
                    adoptBelief("BELIEF(messageNotSent(" + performative + "," + agt + "," + content + "))");
                }
                return sent;
            } catch (Throwable th) {
                th.printStackTrace();
            }
        }
        
        return false;
    }
}
