/*									tab:4
 * Copyright (c) 2007 University College Dublin.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL UNIVERSITY COLLEGE DUBLIN BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF 
 * UNIVERSITY COLLEGE DUBLIN HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * UNIVERSITY COLLEGE DUBLIN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND UNIVERSITY COLLEGE DUBLIN HAS NO
 * OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 *
 *
 */

package com.agentfactory.afapl2.core.module;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import com.agentfactory.logic.agent.Module;
import com.agentfactory.logic.lang.FOS;

/**
 *
 * @author  Mauro Dragone
 */
public class EISControlModule extends Module {
    
    protected Map<String, List<FOS>> memories = new HashMap<String,List<FOS>>();
    protected Set<String> disabled = new TreeSet<String>();

    @Override
    public void init() {
        //Initialization code goes here!
    }

    public synchronized void add(String memoryName, FOS fos) {
        List<FOS> memory = memories.get(memoryName);
        if (memory == null) {
            memory = new LinkedList<FOS>();
            memories.put(memoryName, memory);
        }
        //check if is is already in memory
        memory.add(fos);
    }

    public synchronized void remove(String memoryName, FOS fos) {
        List<FOS> memory = memories.get(memoryName);
        if (memory == null) {
            return;
        }
        List<FOS> listToRemove = new LinkedList<FOS>();
        for (FOS belief:memory) {
            if (fos.matches(belief))                
                listToRemove.add(belief);
        }        
        memory.removeAll(listToRemove);
    }
    
    public synchronized List<FOS> getMemory(String memoryName) {        
        List<FOS> list = new LinkedList<FOS>();
        List<FOS> memory = memories.get(memoryName);
        if (memory != null) {
            for (FOS belief:memory) {
                list.add(belief);
            }
        }
        return list;
    }
    
    public boolean isEnabled(String memoryName) {
        return (!disabled.contains(memoryName));
    }
    
    public void setEnabled(String memoryName, boolean isEnabled) {
        if (isEnabled) {
            disabled.remove(memoryName);
        } else {
            disabled.add(memoryName);
        }
    }
    
}
