package com.agentfactory.afapl2.core.perceptor;

import com.agentfactory.logic.agent.Belief;
import com.agentfactory.logic.agent.Perceptor;
import com.agentfactory.platform.core.IMessage;
import com.agentfactory.platform.util.FIPAHelper;

/**
 * Perceptor that receives incoming messages from other agents and generates beliefs about these messages.
 * A request from another agent generates BELIEF(requested_achieve(?request,?agent) where ?request is some action
 * and ?agent is the agent that sent the message.
 *
 * @author Rem Collier
 * @version July 2002
 */
public class MessagePerceptor extends Perceptor {
    /**
     * generates beliefs from messages
     *
     */
    @Override
	public void perceive() {
        StringBuffer buf = null;
        for (IMessage message: agent.getInbox()) {
            buf = new StringBuffer();
            buf.append("sender(").
                append(message.getSender().getName()).
                append(",addresses(");
            
            boolean first = true;
            for (String address: message.getSender().getAddresses()) {
                if (first) {
                    first = false;
                } else {
                    buf.append(",");
                }
                buf.append(address);
            }
            
            buf.append("))");
            
            Belief belief = createBelief("fipaMessage");
            belief.bind("?performative", message.getPerformative().toLowerCase());
            belief.bind("?sender", buf.toString());
            belief.bind("?content", message.getContent());
            adoptBelief(belief);
            
            belief = createBelief("message");
            belief.bind("?performative", message.getPerformative().toLowerCase());
            belief.bind("?sender", FIPAHelper.toFOSString(message.getSender()));
            belief.bind("?content", message.getContent());
            adoptBelief(belief);
        }
    }
}
