/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.debugger;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import com.agentfactory.afapl2.debugger.model.ActionDescription;
import com.agentfactory.afapl2.debugger.model.CommitmentRuleDescription;
import com.agentfactory.afapl2.debugger.model.GoalDescription;
import com.agentfactory.afapl2.debugger.model.PerceptorDescription;
import com.agentfactory.afapl2.debugger.model.PlanDescription;
import com.agentfactory.afapl2.debugger.model.RoleTemplateDescription;
import com.agentfactory.afapl2.interpreter.AbstractAFAPL2Agent;
import com.agentfactory.afapl2.interpreter.control.RoleTemplate;
import com.agentfactory.afapl2.interpreter.mentalState.Action;
import com.agentfactory.afapl2.interpreter.mentalState.CommitmentRule;
import com.agentfactory.afapl2.interpreter.mentalState.Goal;
import com.agentfactory.afapl2.interpreter.mentalState.Plan;
import com.agentfactory.logic.agent.Perceptor;
import com.agentfactory.logic.agent.beliefManagement.IBeliefManager;
import com.agentfactory.logic.agent.beliefManagement.IBeliefManagerRuleSupport;
import com.agentfactory.logic.agent.beliefManagement.IBeliefManagerTemporalSupport;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.lang.TerImplication;
import com.agentfactory.visualiser.impl.DefaultSnapshot;
import com.agentfactory.visualiser.impl.DefaultStateManager;

/**
 *
 * @author remcollier
 */
public class AFAPL2StateManager extends DefaultStateManager {
    @Override
    protected DefaultSnapshot createSnapshot() {
        return new AFAPL2Snapshot();
    }
    
    @Override
    protected void constructSnapshot(DefaultSnapshot shot) {
        super.constructSnapshot(shot);
        
        AbstractAFAPL2Agent agt = (AbstractAFAPL2Agent) agent;
        AFAPL2Snapshot snap = (AFAPL2Snapshot) shot;
        
        snap.ontologies = agt.getOntologyManager().getOntologies();
        
        snap.perceptors = new LinkedList<PerceptorDescription>();
        snap.perceptorBeliefs = new HashMap<String, List<String>>();
        for (Perceptor perceptor:agt.getPerceptors()) {
            snap.perceptors.add(new PerceptorDescription(perceptor));
            List<String> percepts = new LinkedList<String>();
            for (FOS fos: perceptor.getPerceptions()) percepts.add(fos.toString());
            snap.perceptorBeliefs.put(perceptor.getIdentifier().toString(), percepts);
        }

        IBeliefManager manager = agt.getBeliefManager();
        snap.currentBeliefs = new LinkedList<FOS>(manager.getBeliefs());
        snap.temporalBeliefs = new LinkedList<FOS>();
        snap.beliefRules = new LinkedList<TerImplication>();

        // If the belief manager supports temporal beliefs, copy them
        if (IBeliefManagerTemporalSupport.class.isInstance(manager)) {
            snap.temporalBeliefs.addAll(((IBeliefManagerTemporalSupport) manager).getTemporalBeliefs());
        }

        // If the belief manager supports rules, copy them
        if (IBeliefManagerRuleSupport.class.isInstance(manager)) {
            snap.beliefRules.addAll(((IBeliefManagerRuleSupport) manager).getRules());
        }
        
        snap.roles = new LinkedList<RoleTemplateDescription>();
        for (RoleTemplate role: agt.getRoleLibrary().getRoles()) {
            snap.roles.add(new RoleTemplateDescription(role));
        }
        
        snap.goals = new LinkedList<GoalDescription>();
        for (Goal goal: agt.getGoalManager().getGoals()) {
            snap.goals.add(new GoalDescription(goal));
        }
        
        snap.setCommitments(agt.getCommitmentManager().getCommitments());
        snap.commitmentRules = new LinkedList<CommitmentRuleDescription>();
        for (CommitmentRule rule:agt.getCommitmentManager().getCommitmentRules()) {
            snap.commitmentRules.add(new CommitmentRuleDescription(rule, snap.commitments, agt.getCommitmentManager().getCommitments()));
        }
        
        snap.plans = new LinkedList<PlanDescription>();
        for (Plan plan: agt.getPlanLibrary().getPlans()) {
            snap.plans.add(new PlanDescription(plan));
        }
        
        snap.actions = new LinkedList<ActionDescription>();
        for (Action action: agt.getActions().values()) {
            snap.actions.add(new ActionDescription(action));
        }
        snap.actionSchedule = agt.getActionSchedule();
        snap.completedItems = agt.getCompletedItems();
    }
}
