/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.debugger.inspector;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.MouseListener;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;

import com.agentfactory.afapl2.debugger.AFAPL2Snapshot;
import com.agentfactory.afapl2.debugger.model.RoleTemplateDescription;
import com.agentfactory.visualiser.core.ISnapshot;
import com.agentfactory.visualiser.impl.InspectorPanel;

/**
 *
 * @author remcollier
 */
public class RolesPanel extends JPanel implements InspectorPanel {
    private AFAPL2Snapshot snapshot;
    private JList roles;
    private JList triggers;
    private JList rules;
    private JList instances;
    
    public RolesPanel() {
        setLayout(new BorderLayout());
        
        // Role Template Tab
        roles = new JList(new DefaultListModel());
        roles.addMouseListener(new MouseListener() {
            public void mouseClicked(java.awt.event.MouseEvent mouseEvent) {
                RoleTemplateDescription template = (RoleTemplateDescription) roles.getSelectedValue();
                if (template != null) updateRoleDetails(template);
            }
            
            public void mouseReleased(java.awt.event.MouseEvent mouseEvent) {}
            public void mousePressed(java.awt.event.MouseEvent mouseEvent) {}
            public void mouseExited(java.awt.event.MouseEvent mouseEvent) {}
            public void mouseEntered(java.awt.event.MouseEvent mouseEvent) {}
        });
        roles.setCellRenderer(new DefaultListCellRenderer() {
            @Override
			public java.awt.Component getListCellRendererComponent(javax.swing.JList list, Object value, int index, boolean iss, boolean chf) {
                JLabel label = (JLabel) super.getListCellRendererComponent(list, value.toString(), index, iss, chf);

                if (value instanceof RoleTemplateDescription) {
                    RoleTemplateDescription description = (RoleTemplateDescription) value;
                    label.setText( description.getIdentifier() );
                }

                return label;
            }
        });
        
        JPanel panel = new JPanel(new BorderLayout());
        JToolBar toolbar = new JToolBar();
        toolbar.add(new JLabel("Roles:"));
        panel.add(BorderLayout.NORTH, toolbar);
        panel.add(BorderLayout.CENTER, new JScrollPane(roles));
        
        JPanel lowerPanel = new JPanel(new GridLayout(2,0));
        
        // Trigger SubPane
        JPanel triggerPanel = new JPanel(new BorderLayout());
        toolbar = new JToolBar();
        toolbar.add(new JLabel("Triggers:"));
        triggers = new JList(new DefaultListModel());
        triggerPanel.add(BorderLayout.NORTH, toolbar);
        triggerPanel.add(BorderLayout.CENTER, new JScrollPane(triggers));
        lowerPanel.add(triggerPanel);
        
        // Rules SubPane
        JPanel rulesPanel = new JPanel(new BorderLayout());
        toolbar = new JToolBar();
        toolbar.add(new JLabel("Commitment Rules:"));
        rules = new JList(new DefaultListModel());
        rulesPanel.add(BorderLayout.NORTH, toolbar);
        rulesPanel.add(BorderLayout.CENTER, new JScrollPane(rules));
        lowerPanel.add(rulesPanel);

        JSplitPane sPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        sPane.setDividerLocation(200);
        sPane.setTopComponent(panel);
        
        // Role Instance Tab
        instances = new JList(new DefaultListModel());
        JPanel left = new JPanel(new BorderLayout());
        toolbar = new JToolBar();
        toolbar.add(new JLabel("Instances:"));
        left.add(BorderLayout.NORTH, toolbar);
        left.add(BorderLayout.CENTER, new JScrollPane(instances));
        
        JPanel basePanel = new JPanel(new BorderLayout());
        basePanel.add(BorderLayout.CENTER, lowerPanel);
        basePanel.add(BorderLayout.WEST, left);
        sPane.setBottomComponent(basePanel);
        this.add(BorderLayout.CENTER, sPane);
    }
    
    public void update(ISnapshot _snapshot) {
        snapshot = (AFAPL2Snapshot) _snapshot;
        
        RoleTemplateDescription selected = (RoleTemplateDescription) roles.getSelectedValue();
        
        DefaultListModel model = (DefaultListModel) roles.getModel();
        model.removeAllElements();
        
        for (RoleTemplateDescription desc: snapshot.getRoles()) {
            model.addElement(desc);
        }
        
        if (selected != null) {
            roles.setSelectedValue(selected, true);
            updateRoleDetails(selected);
        }
    }
    
    private void updateRoleDetails(RoleTemplateDescription template) {
        DefaultListModel model = (DefaultListModel) triggers.getModel();
        model.removeAllElements();
        for (String trigger:template.getTriggers()) {
            model.addElement(trigger);
        }
        
        model = (DefaultListModel) rules.getModel();
        model.removeAllElements();
        for (String rule:template.getCommitmentRules()) {
            model.addElement(rule);
        }
        
        model = (DefaultListModel) instances.getModel();
        model.removeAllElements();
        for (String instance: template.getInstances()) {
            model.addElement(instance);
        }
    }
}
