/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.agentfactory.afapl2.interpreter;

import java.util.LinkedList;
import java.util.List;
import java.util.Observable;

import com.agentfactory.afapl2.interpreter.mentalState.Action;
import com.agentfactory.afapl2.interpreter.mentalState.Commitment;
import com.agentfactory.logic.agent.ActivityIdentifier;
import com.agentfactory.logic.agent.Actuator;
import com.agentfactory.logic.agent.Perceptor;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.lang.LogicFactory;
import com.agentfactory.logic.lang.MalformedLogicException;
import com.agentfactory.logic.lang.TerAND;

/**
 * This AFAPL2 Agent class should be used to create Java-based AFAPL2 agents
 * rather than a script based AFAPL2 Agent. To use this class, simply subclass
 * it and use the helper methods provided below.
 * 
 * @author remcollier
 */
public class AFAPL2Agent extends AbstractAFAPL2Agent {
    protected AFAPL2Agent(String name) {
        super(name);
    }
    
    /**
     * Add an initial belief to the agent.
     * 
     * @param belief
     */
    protected void addInitialBelief(String belief) {
        beliefManager.adopt(new FOS(belief));
    }
    
    /**
     * Add an initial commitment to the agent.
     * 
     * @param to
     * @param time
     * @param maintenance
     * @param activity
     */
    protected void addInitialCommitment(String to, String time, String maintenance, String activity) {
        Object logic;
        
        TerAND condition = null;
        try {
            logic = LogicFactory.create(maintenance);
        } catch (MalformedLogicException ex) {
            throw new RuntimeException("Invalid Maintenance Condition: " + maintenance, ex);
        }
        if (logic instanceof TerAND) {
            condition = (TerAND) logic;
        } else if (logic instanceof FOS) {
            condition = new TerAND();
            condition.add((FOS) logic);
        } else {
            throw new RuntimeException("Invalid Maintenance Condition: " + maintenance);
            
        }
        
        Commitment commitment = new Commitment(new FOS(to), new FOS(time), condition, new FOS(activity));
        commitment.evaluate();
        
        commitmentManager.addInitialCommitment(commitment);
    }
    
    /**
     * Add a perceptor to the agent.
     * 
     * @param perceptor
     */
    protected void addPerceptor(Perceptor perceptor) {
        perceptor.setAgent(this);
        perceptors.add(perceptor);
    }

    /**
     * Add an action to the agent with default cardinality of 0 (infinite concurrent
     * activations).
     * 
     * @param id
     * @param pre
     * @param post
     * @param actuator
     */
    protected void addAction(ActivityIdentifier id, String pre, String post, Actuator actuator) {
        this.addAction(id, pre, post, actuator, 0);
    }
    
    /**
     * Add an action to the agent.
     * 
     * @param id
     * @param preCondition
     * @param postCondition
     * @param actuator
     * @param cardinality
     */
    protected void addAction(ActivityIdentifier id, String preCondition, String postCondition, Actuator actuator, int cardinality) {
        TerAND pre = new TerAND();
        pre.add(new FOS(preCondition));
        
        List<TerAND> post = new LinkedList<TerAND>();
        TerAND pc = new TerAND();
        pc.add(new FOS(postCondition));
        post.add(pc);
        
        actuator.setAgent(this);
        Action action = new Action(id, pre, post, actuator, cardinality);
        actions.put(action.getIdentifier().getSignature(), action);
    }

    public void update(Observable o, Object arg) {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
