/*
 * Role.java
 *
 * Created on 28 January 2005, 19:18
 */

package com.agentfactory.afapl2.interpreter.control;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;

import com.agentfactory.afapl2.interpreter.mentalState.Commitment;
import com.agentfactory.afapl2.interpreter.mentalState.CommitmentRule;
import com.agentfactory.afapl2.interpreter.mentalState.Role;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.reasoner.SubstitutionSet;
import java.util.List;
import java.util.Map;
/**
 *
 * @author  Administrator
 */
public class RoleTemplate implements Serializable {
    private FOS identifier;
    private FOS superClass;
    private Map<String,String> modules;
    private List<String> actuators;
    private List<String> perceptors;
    private List<CommitmentRule> commitmentRules;
    private List<Commitment> commitments;
    private List<Role> instances;
    private List<Object> triggers;
    
    public RoleTemplate(FOS identifier) {
        this.identifier = identifier;
        this.commitmentRules = new ArrayList<CommitmentRule>();
        this.actuators = new ArrayList<String>();
        this.perceptors = new ArrayList<String>();
        this.modules = new HashMap<String,String>();
        this.instances = new ArrayList<Role>();
        this.triggers = new ArrayList<Object>();
        this.commitments = new ArrayList<Commitment>();
    }
    
    public void setSuperClass(FOS superClass) {
        this.superClass = superClass;
    }
    
    public FOS getSuperClass() {
        return superClass;
    }
    
    public void addActuator(String name) {
        actuators.add(name);
    }

    public void addPerceptor(String name) {
        perceptors.add(name);
    }

    public void addModule(String id, String type) {
        modules.put(id, type);
    }

    public void addCommitmentRule(CommitmentRule rule) {
        commitmentRules.add(rule);
    }
    
    public void addCommitment(Commitment commitment) {
        commitments.add(commitment);
    }
    
    public void addTrigger(Object trigger) {
        triggers.add(trigger);
    }
    
    @Override
	public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("ROLE " + identifier.toString());
        buf.append('{');
        buf.append("\n");
        for(int i=0; i<triggers.size();i++){
            buf.append("TRIGGER "+triggers.get(i)+";\n");
        }
        for(int i=0; i<commitments.size();i++){
            buf.append(commitments.get(i)+";\n");
        }
        for(int i=0; i<commitmentRules.size();i++){
            buf.append(commitmentRules.get(i)+";\n");
        }
        buf.append('}');        
        buf.append("\n");
        return buf.toString();
    }
    
    public FOS getIdentifier() {
        return identifier;
    }
    
    public List<CommitmentRule> getCommitmentRules() {
        return commitmentRules;
    }
    
    public List<Commitment> getCommitments() {
        return commitments;
    }
    
    public List<String> getActuators() {
        return actuators;
    }
    
    public List<String> getPerceptors() {
        return perceptors;
    }
    
    public List<Object> getTriggers() {
        return triggers;
    }
    
    private Role getInstance(FOS identifier) {
        Role role = null;
        boolean found = false;
        
        int index = 0;
        while ((index < instances.size()) && !found) {
            role = instances.get(index);
            found = role.getIdentifier().equals(identifier);
            index++;
        }
        
        if (!found) return null;
        return role;
    }
    
    public Role instantiate(SubstitutionSet binding, RoleLibrary library) {
        FOS id = identifier.copy().apply(binding);
        if (this.getInstance(id) != null) return null;
        
        Role role = new Role(id);
        for (int i = 0; i < perceptors.size(); i++) {
            role.addPerceptor((String) perceptors.get(i));
        }
        for (int i = 0; i < actuators.size(); i++) {
            role.addActuator((String) actuators.get(i));
        }
        for (int i = 0; i < commitmentRules.size(); i++) {
            role.addCommitmentRule(((CommitmentRule) commitmentRules.get(i)).apply(binding));
        }
        for (int i = 0; i < commitments.size(); i++) {
            role.addCommitment(((Commitment) commitments.get(i)).apply(binding));
        }
        
        if (superClass != null) {
            RoleTemplate template = library.getRoleTemplate(superClass);
            if (template == null) {
                System.out.println("no such role: " + superClass + " in: " + id);
            }
            role.merge(template.createParent(binding, library));
        }
        
        instances.add(role);
        return role;
    }

    private Role createParent(SubstitutionSet binding, RoleLibrary library) {
        FOS id = identifier.copy().apply(binding);
        Role role = new Role(id);
        for (int i = 0; i < perceptors.size(); i++) {
            role.addPerceptor((String) perceptors.get(i));
        }
        for (int i = 0; i < actuators.size(); i++) {
            role.addActuator((String) actuators.get(i));
        }
        for (int i = 0; i < commitmentRules.size(); i++) {
            role.addCommitmentRule(((CommitmentRule) commitmentRules.get(i)).apply(binding));
        }
        
        if (superClass != null) {
            RoleTemplate template = library.getRoleTemplate(superClass);
            if (template == null) {
                System.out.println("no such role: " + superClass + " in: " + id);
            }
            role.merge(template.createParent(binding, library));
        }
        
        return role;
    }

    public void destroy(Role role) {
        instances.remove(role);
    }
    
    public List<Role> getInstances() {
        return instances;
    }
}
