package com.agentfactory.agentspeak.interpreter;

import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;

import com.agentfactory.agentspeak.compiler.AgentSpeakParser;
import com.agentfactory.agentspeak.compiler.CodeGeneratorVisitor;
import com.agentfactory.agentspeak.compiler.InvalidParentNameException;
import com.agentfactory.agentspeak.compiler.SimpleNode;
import com.agentfactory.agentspeak.compiler.TemplateVisitor;
import com.agentfactory.agentspeak.interpreter.mentalstate.ASEvent;
import com.agentfactory.agentspeak.interpreter.mentalstate.PlanRule;
import com.agentfactory.clf.interpreter.Action;
import com.agentfactory.clf.interpreter.Module;
import com.agentfactory.clf.interpreter.Sensor;
import com.agentfactory.clf.lang.IMPLIES;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.platform.core.AgentCreationException;

public class ScriptBasedAgentSpeakAgent extends AbstractAgentSpeakAgent {

	public ScriptBasedAgentSpeakAgent(String name) {
		super(name);
		eventQueue.add(new ASEvent(ASEvent.ADDITION, (Predicate) Utilities.factory.createFormula("initialized")));
	}

	public void construct(String design) throws AgentCreationException {
		List<SimpleNode> programs = new LinkedList<SimpleNode>();
		try {
			TemplateVisitor tVisitor = new TemplateVisitor();

			InputStream in = null;
			SimpleNode node = null;
			AgentSpeakParser parser = null;
			String parent = null;
			String current = "/" + design;
			while (!current.equals(TemplateVisitor.NO_EXTENDS)) {
				in = AgentSpeakParser.class.getResourceAsStream(current);
				if (in == null)
					throw new InvalidParentNameException("no such parent: "
							+ current + " in file: " + parent);

				parser = new AgentSpeakParser(in);
				node = parser.Start();
				programs.add(0, node);

				parent = current;
				current = (String) tVisitor.visit(node, current);
			}

			CodeGeneratorVisitor cgv = new CodeGeneratorVisitor();
			for (SimpleNode n : programs) {
				cgv.visit(n, null);
			}

			if (cgv.isAbstract) throw new AgentCreationException("Agent type is abstract: cannot instantiate");
			
			if (cgv.hasAbstractPlans()) throw new AgentCreationException("Agent type contains abstract plans: cannot instantiate");

			if (!parser.errors.isEmpty()) {
				throw new RuntimeException("ERROR: AgentSpeak program does not compile: " + design);
			}
			
			for (IMPLIES inference:cgv.inferences) {
				addInference(inference);
			}
			
			for (Predicate id: cgv.actions.keySet()) {
				Action action = (Action) Class.forName(cgv.actions.get(id)).newInstance();
				action.setId(id);
				action.setAgent(this);
				actionMap.put(Utilities.createSignature(id), action);
			}
			
			for (Predicate id: cgv.sensors.keySet()) {
				Sensor sensor = (Sensor) Class.forName(cgv.sensors.get(id)).newInstance();				
				sensor.setId(id);
				sensor.setAgent(this);
				sensorMap.put(Utilities.createSignature(id), sensor);
			}
			
			for (Predicate id: cgv.modules.keySet()) {
				addModule(id, (Module) Class.forName(cgv.modules.get(id)).newInstance());
			}

			for (String label: cgv.partialPlans.keySet()) {
				partialPlanMap.put(label, cgv.partialPlans.get(label));
			}
			
			for (PlanRule rule: cgv.rules) {
				addPlanRule(rule);
			}

			for (Predicate belief: cgv.beliefs) {
				this.addBelief(belief);
			}
		} catch (Throwable e) {
//			e.printStackTrace();
			System.out.println("Parse Exception for: " + this.name + ": " + e.getMessage());
		}
	}
}
