package com.agentfactory.astr.compiler;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.agentfactory.astr.interpreter.ASEvent;
import com.agentfactory.astr.interpreter.ASTRProgram;
import com.agentfactory.astr.interpreter.ASTRRule;
import com.agentfactory.astr.interpreter.PartialPlan;
import com.agentfactory.astr.interpreter.PlanRule;
import com.agentfactory.astr.interpreter.Utilities;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.AND;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.IMPLIES;
import com.agentfactory.clf.lang.ITerm;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.plans.IPlanStep;

public class CodeGeneratorVisitor implements ASTRParserVisitor {
	public Map<Predicate, String> actions = new HashMap<Predicate, String>();
	public Map<Predicate, String> sensors = new HashMap<Predicate, String>();
	public Map<Predicate, String> modules = new HashMap<Predicate, String>();
	public Map<String, PlanRule> plans = new HashMap<String, PlanRule>();
	public Map<String, PartialPlan> partialPlans = new HashMap<String, PartialPlan>();
	private Map<String, Boolean> abstractPlans = new HashMap<String, Boolean>();
	public List<PlanRule> rules = new LinkedList<PlanRule>();
	public List<Predicate> beliefs = new LinkedList<Predicate>();
	public List<IMPLIES> inferences = new LinkedList<IMPLIES>();
	public boolean isAbstract = false;
	public List<ASTRProgram> programs = new LinkedList<ASTRProgram>();
	public String name;
	private static int counter = 0;
	
	public boolean hasAbstractPlans() {
		return !abstractPlans.isEmpty();
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public Object visit(SimpleNode node, Object data) {
		PredicateBuilderVisitor visitor = new PredicateBuilderVisitor();
		if (node.toString().equals("TemplateExpression")) {
			isAbstract = "#abstract".equals(node.jjtGetValue());
		} if (node.toString().equals("SensorExpression")) {
			String predicate = (String) visitor.visit((SimpleNode) node.jjtGetChild(0), null);
			String cname = (String) ((SimpleNode) node.jjtGetChild(1)).jjtGetValue();
			if (cname != null) {
				sensors.put((Predicate) Utilities.factory.createFormula(predicate), cname);
			}
		} else if (node.toString().equals("ActionExpression")) {
			String predicate = (String) visitor.visit((SimpleNode) node.jjtGetChild(0), null);
			String cname = (String) ((SimpleNode) node.jjtGetChild(1)).jjtGetValue();
			if (cname != null) {
				actions.put((Predicate) Utilities.factory.createFormula(predicate), cname);
			}
		} else if (node.toString().equals("ModuleExpression")) {
			String predicate = (String) visitor.visit((SimpleNode) node.jjtGetChild(0), null);
			String cname = (String) ((SimpleNode) node.jjtGetChild(1)).jjtGetValue();
			if (cname != null) {
				modules.put((Predicate) Utilities.factory.createFormula(predicate), cname);
			}
		} else if (node.toString().equals("InferenceExpression")) {
			AND and = (AND) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null);
			String rhs = (String) new PredicateBuilderVisitor().visit((SimpleNode) node.jjtGetChild(1), null);
			inferences.add(new IMPLIES(and, CoreUtilities.factory.createFormula(rhs)));
		} else if (node.toString().equals("AbstractRuleExpression")) {
			String label = (String) new PredicateBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null);
			abstractPlans.put(label, "#partial".equals(node.jjtGetValue()));
		} else if (node.toString().equals("UnlabelledRuleExpression")) {
			int i=0;
			String label = "rule" + counter++;
			SimpleNode eventNode = (SimpleNode) node.jjtGetChild(i++);
			int type = (eventNode.jjtGetValue().equals("+")) ? ASEvent.ADDITION : ASEvent.REMOVAL;
			SimpleNode evt = (SimpleNode) eventNode.jjtGetChild(0);
			String predicateEvent = "";
			if (evt.toString().equals("Goal")) {
				predicateEvent += "!";
				evt = (SimpleNode) evt.jjtGetChild(0);
			}
			predicateEvent += (String) visitor.visit(evt, null);
			AND and = (AND) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(i++), null);
			AND condition = new AND(new IFormula[] {Utilities.factory.createFormula("true") });
			List<IPlanStep> plan = (List<IPlanStep>) new PlanBuilderVisitor().visit((SimpleNode) node.jjtGetChild(i++), null);
			PlanRule rule = new PlanRule( (Predicate) Utilities.factory.createFormula(label),
									new ASEvent(type, Utilities.factory.createFormula(predicateEvent)),
									and, condition, plan);
			plans.put(label, rule);
			rules.add(rule);
		} else if (node.toString().equals("LabelledRuleExpression")) {
			SimpleNode typeNode = (SimpleNode) node.jjtGetChild(0);
			if (typeNode.toString().equals("Full")) {
				String label = (String) new PredicateBuilderVisitor().visit((SimpleNode) typeNode.jjtGetChild(0), null);
				
//				System.out.println("Full Rule");
				if ("#overrides".equals(node.jjtGetValue())) {
					if (plans.containsKey(label)) {
						rules.remove(plans.get(label));
					} else {
						throw new IllegalOverrideException("You cannot override the plan labelled: @" + label + " as it does not exist.");
					}
				} else if ("#implements".equals(node.jjtGetValue())) {
					Boolean b = abstractPlans.get(label);
					if (b != null) {
						if (b.booleanValue()) {
							throw new IllegalImplementsException("You cannot implement the plan labelled: @" + label + " as has not been declared partial.");
						} else {
							abstractPlans.remove(label);
						}
					} else {
						throw new IllegalImplementsException("You cannot implement the plan labelled: @" + label + " as has not been declared abstract.");
					}
				} else {
					if (plans.containsKey(label)) {
						throw new IllegalOverrideException("To override the plan labelled: @" + label + " you must use the #overrides tag.");
					}
					
					if (abstractPlans.containsKey(label)) {
						throw new IllegalOverrideException("You cannot override the plan labelled: @" + label + " as it has already been declared abstract.");
					}
				}

				SimpleNode eventNode = (SimpleNode) typeNode.jjtGetChild(1);
				int type = (eventNode.jjtGetValue().equals("+")) ? ASEvent.ADDITION : ASEvent.REMOVAL;
				SimpleNode evt = (SimpleNode) eventNode.jjtGetChild(0);
				String predicateEvent = "";
				if (evt.toString().equals("Goal")) {
					predicateEvent += "!";
					evt = (SimpleNode) evt.jjtGetChild(0);
				}
				predicateEvent += (String) visitor.visit(evt, null);
				AND and = (AND) new SentenceBuilderVisitor().visit((SimpleNode) typeNode.jjtGetChild(2), null);
				AND condition = null;
				List<IPlanStep> plan = null;

				if (typeNode.jjtGetNumChildren() == 5) {
					condition = (AND) new SentenceBuilderVisitor().visit((SimpleNode) typeNode.jjtGetChild(3), null);
					plan = (List<IPlanStep>) new PlanBuilderVisitor().visit((SimpleNode) typeNode.jjtGetChild(4), null);
				} else {
					condition = new AND(new IFormula[] {Utilities.factory.createFormula("true") });
					plan = (List<IPlanStep>) new PlanBuilderVisitor().visit((SimpleNode) typeNode.jjtGetChild(3), null);
				}
				
				PlanRule rule = new PlanRule( (Predicate) Utilities.factory.createFormula(label),
										new ASEvent(type, Utilities.factory.createFormula(predicateEvent)),
										and, condition, plan);
				plans.put(label, rule);
	//			System.out.println("adding rule: " + rule.toString());
				rules.add(rule);
			} else {
				String label = (String) new PredicateBuilderVisitor().visit((SimpleNode) typeNode.jjtGetChild(0), null);
				
				if (plans.containsKey(label) || partialPlans.containsKey(label)) {
					throw new IllegalPartialPlanException("You cannot declare the partial plan labelled: @" + label + " the plan has already been declared.");
				}
				
				if ("#overrides".equals(node.jjtGetValue())) {
					if (!partialPlans.containsKey(label)) {
						throw new IllegalOverrideException("You cannot override the partial plan labelled: @" + label + " as it does not exist.");
					}
				} else if ("#implements".equals(node.jjtGetValue())) {
					Boolean b = abstractPlans.get(label);
					if (b == null) {
						throw new IllegalImplementsException("You cannot implement the plan labelled: @" + label + " as has not been declared abstract.");
					}
					
					if (!b.booleanValue()) {
						throw new IllegalImplementsException("You cannot implement the plan labelled: @" + label + " as has been declared abstract but not declared partial.");
					}
					
					abstractPlans.remove(label);
				} else {
					if (abstractPlans.containsKey(label)) {
						throw new IllegalPartialPlanException("You cannot declare the partial plan labelled: @" + label + " the plan has already been declared as abstract.");
					}
				}
				
				List<IPlanStep> plan = (List<IPlanStep>) new PlanBuilderVisitor().visit((SimpleNode) typeNode.jjtGetChild(1), null);
				PartialPlan partial = new PartialPlan( (Predicate) Utilities.factory.createFormula(label), plan);
				partialPlans.put(CoreUtilities.createSignature(partial.getLabel()), partial);
				
			}
		} else if (node.toString().equals("Predicate")) {
			String predicate = (String) new PredicateBuilderVisitor().visit(node, null);
			beliefs.add((Predicate) Utilities.factory.createFormula(predicate +"[source(self)]"));
		} else if (node.toString().equals("IncludeExpression")) {
			System.out.println("In include expression");
			// IGNORE: This should never be called
		} else if (node.toString().equals("FuncExpression")) {
			List<ASTRRule> rules = new LinkedList<ASTRRule>();
			List<ITerm> arguments = new LinkedList<ITerm>();
			SimpleNode function = (SimpleNode) node.jjtGetChild(0);
			Predicate functiondec = (Predicate) CoreUtilities.factory.createFormula((String) visitor.visit(function, null));
			for (int i = 0; i < functiondec.size(); i++) {
				arguments.add(functiondec.termAt(i));   
            }
			String funName = CoreUtilities.createSignature(functiondec);
			// get name of function
			for (int i = 1; i < node.jjtGetNumChildren(); i++) {// for each rule load and add to trprog
				ASTRRule r = (ASTRRule)visit(((SimpleNode)node.jjtGetChild(i)), data);
				rules.add(r);
			}
			ASTRProgram prog = new ASTRProgram(funName, rules, arguments);
			if(node.jjtGetValue() != null){
				if(node.jjtGetValue().equals("#abstract")){
					prog.setAbstract(true);
					if(prog.getRules().size() >0)
						System.err.println("Function "+prog.getName()+" in "+name+ " is declared abstract and should not have any rules");
				} else {
					if(prog.getRules().size() == 0)
						System.err.println("Function "+prog.getName()+" in "+name+ " should be declared abstract");
				}
				if(node.jjtGetValue().equals("#override"))
					prog.setOverride(true);
				else if(node.jjtGetValue().equals("#implements")){
					prog.setImplements(true);
				}
			} else{
				if(prog.getRules().size() == 0)
					System.err.println("Function "+prog.getName()+" in "+name+ " should be declared abstract");

			}
			programs.add(prog);
		} else if(node.toString().equals("RuleExpression")){
			AND and = (AND) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null);
			String predicateAction = (String) visitor.visit((SimpleNode) node.jjtGetChild(1), null);
			IFormula act = CoreUtilities.factory.createFormula(predicateAction);
			ASTRRule rule = new ASTRRule(and, act);
			return rule;
		} else {
			for (int i=0; i<node.jjtGetNumChildren(); i++) {
				visit((SimpleNode) node.jjtGetChild(i), data);
			}
		}
		return null;
	}

}
