package com.agentfactory.astr.compiler;


import static org.junit.Assert.fail;

import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agentfactory.astr.compiler.ASTRError;
import com.agentfactory.astr.compiler.ASTRParser;
import com.agentfactory.astr.compiler.CodeGeneratorVisitor;
import com.agentfactory.astr.compiler.InvalidParentNameException;
import com.agentfactory.astr.compiler.SimpleNode;
import com.agentfactory.astr.compiler.TemplateVisitor;
import com.agentfactory.astr.interpreter.ASTRProgram;
import com.agentfactory.astr.interpreter.PlanRule;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.IMPLIES;

public class CompilerTest {

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testCompiler() {
		List<SimpleNode> programs = new LinkedList<SimpleNode>();
		try {
			String url = "/com/agentfactory/astr/compiler/test.aspeak";
			TemplateVisitor tVisitor = new TemplateVisitor();

			InputStream in = null;
			SimpleNode node = null;
			ASTRParser parser = null;
			String parent = null;
			String current = url;
			while (!current.equals(TemplateVisitor.NO_EXTENDS)) {
				System.out.println("Parsing: " + current);
				in = ASTRParser.class.getResourceAsStream(current);
				if (in == null)
					throw new InvalidParentNameException("no such parent: "
							+ current + " in file: " + parent);

				parser = new ASTRParser(in);
				node = parser.Start();
				programs.add(0, node);

				parent = current;
				current = (String) tVisitor.visit(node, current);
			}

			CodeGeneratorVisitor iVisitor = new CodeGeneratorVisitor();
			for (SimpleNode n : programs) {
				iVisitor.visit(n, null);
			}

			if (iVisitor.isAbstract) System.out.println(">>>>>>>>>> This is abstract <<<<<<<<<<");
			
			if (iVisitor.hasAbstractPlans()) System.out.println(">>>>>>>>>> Has abstract plans <<<<<<<<<<");
			for (IMPLIES inference : iVisitor.inferences) {
				System.out.println("inference:"
						+ CoreUtilities.presenter.toString(inference));
			}
			for (PlanRule rule : iVisitor.rules) {
				System.out.println("rule:" + rule);
			}
			for (String label: iVisitor.partialPlans.keySet()) {
				System.out.println("partial plan:" + label);
				System.out.println(iVisitor.partialPlans.get(label));
			}
			
			for (ASTRProgram prog : iVisitor.programs) {
				System.out.println("prog:" + prog);
			}
			
			for (ASTRError error : parser.errors) {
				System.out.println(error.issue);
			}
			System.out.println("Thank you.");
		} catch (Exception e) {
			e.printStackTrace();
			fail("failed to run");
		}
	}

}
