package com.agentfactory.astr.interpreter;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.plans.BasicStep;
import com.agentfactory.clf.plans.IPlanStep;
import com.agentfactory.clf.plans.PlanStack;
import com.agentfactory.clf.reasoner.Bindings;
import com.agentfactory.clf.reasoner.QueryResult;

public class ASTRBasicStep extends BasicStep {
	public ASTRBasicStep(IFormula step) {
		super(step);
	}
	
	@Override
	public void handle(AbstractLanguageAgent ala, PlanStack planStack) {
		AbstractASTRAgent agent = (AbstractASTRAgent) ala;
		Intention intention = (Intention) planStack;
		IFormula activity = step.apply(intention.bindings());
		if (TestFormula.class.isInstance(activity)) {
			QueryResult result = agent.queryAll(((TestFormula) activity).formula());
			if (result.found) {
				intention.updateBindings(result.bindings.get(0));
//				agent.logAction(CoreUtilities.presenter.toString(activity) + " {SUCCESS]");
			}
		} else if (StateUpdateFormula.class.isInstance(activity)) {
			StateUpdateFormula formula = (StateUpdateFormula) activity;
			Predicate selfSource = (Predicate) Utilities.factory.createFormula("source(self)");
			if (formula.updateType() == StateUpdateFormula.ADDITION) {
				if (Predicate.class.isInstance(formula.formula())) {
					Predicate predicate = (Predicate) formula.formula().apply(new Bindings());
					predicate.getLabels().add(selfSource);
					agent.addBelief(predicate, null);
					agent.logAction("+" + CoreUtilities.presenter.toString(predicate) + " {SUCCESS]");
				}
			} else if (formula.updateType() == StateUpdateFormula.REMOVAL) {
				if (Predicate.class.isInstance(formula.formula())) {
					Predicate predicate = (Predicate) formula.formula().apply(new Bindings());
					predicate.getLabels().add(selfSource);
					agent.removeBelief(predicate, null);
					agent.logAction("-" + CoreUtilities.presenter.toString(predicate) + " {SUCCESS]");
				}
			}
		} else if (AchievementFormula.class.isInstance(activity)) {
			agent.addEvent(new ASEvent(ASEvent.ADDITION, activity, intention));
			intention.suspend();
		} else {
			super.handle(agent, planStack);
		}
	}
	
	public String toString() {
		return toString("");
	}
	
	public String toString(String tabs) {
		return tabs + Utilities.presenter.toString(step);
	}
	
	public IPlanStep apply(Bindings set) {
		try {
			return new ASTRBasicStep(step.apply(set));
		} catch (RuntimeException e) {
			System.out.println("WARNING: Error for step: " + CoreUtilities.presenter.toString(step) + " when applying: " + CoreUtilities.presenter.toString(set));
			throw e;
		}
	}
	
}
