package com.agentfactory.astr.interpreter;

import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.reasoner.Bindings;

public class ASTRRule {
	private IFormula condition;
	private IFormula action;
	
	public ASTRRule(IFormula condition, IFormula action){
		this.condition = condition;
		this.action = action;
	}
	
	public ASTRRule(String Rule){
		String[] arr = Rule.split("->");
		for (String string : arr) {
			System.out.println(string);
		}
		
		
		this.condition = CoreUtilities.factory.createFormula(arr[0]);
		this.action = CoreUtilities.factory.createFormula(arr[1]);
	}
	/**
	 * Returns the condition which must be satisfied to fire the rule.
	 * @return The condition to be queried by the interpreter.
	 */
	public IFormula getCondition(){
		return condition;
	}
	/**
	 * Returns the action which performed in the event the rule is satisfied.
	 * @return The action which the interpreter performs.
	 */
	public IFormula getAction(){
		return action;
	}
	/**
	 * Produces string representation of the rule.
	 */
	public String toString(){
		String ret = CoreUtilities.presenter.toString(condition) + "->" + CoreUtilities.presenter.toString(action);
		return ret;
	}
	
	/**
	 * Applies a set of bindings to the parameters of the rule
	 * @param A set of Bindings
	 * @return A new TRRule bound to the set provided
	 */
	public ASTRRule apply(Bindings set){
		return new ASTRRule(condition.apply(set), action.apply(set));
	}
	
}
