package com.agentfactory.clf.interpreter;

import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.platform.core.AgentCreationException;
import com.agentfactory.platform.core.DuplicateAgentNameException;
import com.agentfactory.platform.core.NoSuchArchitectureException;
import com.agentfactory.platform.util.FIPAHelper;
import com.agentfactory.service.ams.AgentDetails;
import com.agentfactory.service.ams.AgentManagementService;

public class AMSModule extends Module {
	AgentManagementService service;
	
	public AMSModule() {
		addAction("setup", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				agent.bindToPlatformService(AgentManagementService.NAME);
				service = (AgentManagementService) agent.getPlatformService(AgentManagementService.NAME);
				return true;
			}
		});
		addAction("create(?name, ?design)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String name = CoreUtilities.presenter.toString(activity.termAt(0));
				String design = this.termAsString(activity, 1);
				try {
					service.createAgent(name, design);
				} catch (NoSuchArchitectureException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					return false;
				} catch (DuplicateAgentNameException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					return false;
				} catch (AgentCreationException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					return false;
				}
				return true;
			}
		});
		addAction("terminate(?name)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String name = CoreUtilities.presenter.toString(activity.termAt(0));
				service.terminateAgent(name);
				return true;
			}
		});
		addAction("suspend(?name)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String name = CoreUtilities.presenter.toString(activity.termAt(0));
				service.suspendAgent(name);
				return true;
			}
		});
		addAction("resume(?name)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String name = CoreUtilities.presenter.toString(activity.termAt(0));
				service.resumeAgent(name);
				return true;
			}
		});
		addSensor("agents", new Sensor() {
			@Override
			public void perceive() {
				if (service == null) return;
				for (String name: service.getAgentNames()) {
					if (name.equals(agent.getName())) continue;
					AgentDetails agt = service.getAgentDetails(name);
					if (agt == null) continue;
					addBelief("localAgent(" + name + ")");
					addBelief("agentType(" + name + "," + agt.type + ")");
					addBelief("agentState(" + name + "," + agt.state + ")");
					addBelief(FIPAHelper.toFOSString(agt.agentID));
				}
				
			}
		});
	}
}
