package com.agentfactory.clf.interpreter;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.lang.Variable;
import com.agentfactory.clf.reasoner.Bindings;


public abstract class ActiveAction extends Action {
	/**
	 * This method should not be called for active actions. If it is, then
	 * the default behaviour is to pass in an empty list of variables and
	 * a new set of bindings that will never be added to the plan stack. 
	 * 
	 */
	public boolean execute(Predicate activity) {
		return execute(activity, new LinkedList<Variable>(), new Bindings());
	}
	
	/**
	 * This method replaces the standard action method. The key difference
	 * is that the a set of Bindings is passed and new bindings can be added
	 * as required by the action implementation. these bindings are then
	 * merged with the current set of bindings associated with the agent.
	 * 
	 * upon completion, the binding set is filtered by the variable list so
	 * unspecified bindings are ignored (e.g. if you allow only three bindings
	 * then only three bindings will be passed through the filter).
	 * 
	 * @param activity the invoked activity
	 * @param variables a list of the variables that can be set
	 * @param bindings the bindings set
	 * @return true if the action was successful, false otherwise
	 * 
	 */
	public abstract boolean execute(Predicate activity, List<Variable> variables, Bindings bindings);
}
