package com.agentfactory.clf.interpreter;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.platform.core.AgentID;
import com.agentfactory.platform.util.FIPAHelper;
import com.agentfactory.service.df.DFAgentDescription;
import com.agentfactory.service.df.DirectoryFacilitatorService;
import com.agentfactory.service.df.ServiceDescription;

public class DFSModule extends Module {
	private DirectoryFacilitatorService service;
	
	public DFSModule() {
		this.addAction("setup", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				agent.bindToPlatformService(DirectoryFacilitatorService.NAME);
				service = (DirectoryFacilitatorService) agent.getPlatformService(DirectoryFacilitatorService.NAME);
				return true;
			}
		});
		this.addAction("register", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				service.register(new DFAgentDescription(agent.getAgentID()));
				return true;
			}
		});
		this.addAction("deregister", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				service.deregister(agent.getAgentID());
				return true;
			}
		});
		
		this.addAction("addService(?name, ?type, ?protocols, ?ontolgoies, ?langauges, ?properties)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String name = termAsString(activity, 0);
				String type = termAsString(activity, 1);
				Set<String> protocols = termAsSet(activity, 2);
				Set<String> ontologies = termAsSet(activity, 3);
				Set<String> languages = termAsSet(activity, 4);
				Map<String, String> properties = termAsMap(activity, 5);
				
				// Assume the first item in the returned list
				List<DFAgentDescription> list = service.get(agent.getAgentID());
				if (list.isEmpty()) {
					addBelief("dfsError(notRegistered)");
					return false;
				}
				
				list.get(0).addService(new ServiceDescription(name, type, protocols, ontologies, languages, agent.getAgentID(), properties));

				return true;
			}
		});
		this.addAction("removeService(?name)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String name = termAsString(activity, 0);
				
				// Assume the first item in the returned list
				List<DFAgentDescription> list = service.get(agent.getAgentID());
				if (list.isEmpty()) {
					addBelief("dfsError(notRegistered)");
					return false;
				}
				
				list.get(0).removeService(name);

				return true;
			}
		});

		this.addSensor("serviceDescriptions", new Sensor() {
			@Override
			public void perceive() {
				if (service == null) {
					addBelief("dfsError(notConnected)");
					return;
				}
				
				List<DFAgentDescription> list = service.search(new DFAgentDescription(new AgentID(".*", new LinkedList<String>(), new LinkedList<String>())));
				for (DFAgentDescription desc :list) {
					addBelief("dfsService(" + FIPAHelper.toFOSString(desc.agentID) + ")");
				}
			}
		});
	}
	

}
