package com.agentfactory.clf.lang;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import com.agentfactory.clf.reasoner.Bindings;

public class OR implements IFormula, Iterable<IFormula> {
    private List<IFormula> formulae;
    
    public OR(List<IFormula> formulae) {
        this.formulae = formulae;
    }
    
    @Override
    public IFormula apply(Bindings b) {
        List<IFormula> xs = new ArrayList<IFormula>(formulae.size());
        for (IFormula f : formulae) xs.add(f.apply(b));
        return new OR(xs);
    }

    @Override
    public List<IFormula> asList() {
        List<IFormula> xs = new ArrayList<IFormula>();
        for (IFormula f : formulae) xs.addAll(f.asList());
        return xs;
    }

    @Override
    public Set<Variable> getVariables() {
        Set<Variable> r = new HashSet<Variable>();
        for (IFormula f : formulae) r.addAll(f.getVariables());
        return r;
    }

    @Override
    public boolean isFalse() {
        return !isTrue();
    }

    @Override
    public boolean isTrue() {
        for (IFormula f : formulae) if (f.isTrue()) return true;
        return false;
    }

    @Override
    public boolean matches(IFormula formula) {
        if (!(formula instanceof OR)) return false;
        OR other = (OR) formula;
        Iterator<IFormula> i = iterator();
        Iterator<IFormula> j = other.iterator();
        while (i.hasNext() && j.hasNext())
            if (!i.next().matches(j.next())) return false;
        return i.hasNext() == j.hasNext();
    }

    @Override
    public String type() {
        return "raw-logic";
    }

    @Override
    public Iterator<IFormula> iterator() {
        return formulae.iterator();
    }

}
