package com.agentfactory.clf.plans;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.interpreter.Action;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.Predicate;

public class ActionThread implements Runnable {
	Predicate activity;
	AbstractLanguageAgent agent;
	boolean success;
	boolean finished;
	Action action;

	public ActionThread(AbstractLanguageAgent agent, Predicate action) {
		this.activity = action;
		this.agent = agent;
		finished = false;
	}

	@Override
	public void run() {
		action = agent.getActionMap().get(CoreUtilities.createSignature(activity));
		if (action == null) {
			action = agent.getModuleLevelAction(activity);
			if (action == null) {
				agent.noSuchAction(activity);
				setSuccess(false);
				setFinished(true);
			}
		}
		
		boolean result = action.execute(activity);

		setSuccess(result);
		setFinished(true);
	}

	public synchronized void setFinished(boolean finished) {
		this.finished = finished;
	}

	public synchronized boolean isFinished() {
		return finished;
	}

	public synchronized void setSuccess(boolean success) {
		this.success = success;
	}

	public synchronized boolean isSuccessful() {
		return success;
	}
	
	public void finish(PlanStack planStack) {
		// finished, so check outcome and drop the step
		if (!isSuccessful()) {
			planStack.fail();
		}
		
		agent.logAction(CoreUtilities.presenter.toString(activity));
	}
}
