package com.agentfactory.clf.plans;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.interpreter.Action;
import com.agentfactory.clf.interpreter.ActiveAction;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.Function;
import com.agentfactory.clf.lang.ITerm;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.lang.Variable;
import com.agentfactory.clf.reasoner.Bindings;

public class AssignmentStep implements IPlanStep {
	private Variable variable;
	private ITerm expression;
	private ActiveActionThread thread;
	private boolean durative;
		
	public AssignmentStep(Variable variable, ITerm expression, boolean durative) {
		this.variable = variable;
		this.expression = expression;
		this.durative = durative;
	}
	
	@Override
	public IPlanStep apply(Bindings set) {
		return new AssignmentStep(variable, expression.apply(set), durative);
	}

	public String toString() {
		return toString("");
	}
	
	public String toString(String tabs) {
		String output = tabs + CoreUtilities.presenter.toString(variable) + "=";
		if (durative) output += "durative(";
		output += CoreUtilities.presenter.toString(expression);
		if (durative) output += ")";
		return output;
	}

	public ITerm variable() {
		return variable;
	}
	
	public ITerm expression() {
		return expression;
	}

	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		// Lazy initialization of action thread (performed the first time
		// the step is handled)
		if (thread == null) {
			ITerm term = expression.apply(planStack.bindings());
			if (Function.class.isInstance(term)) {
				Predicate activity = CoreUtilities.factory.convertToPredicate(term);

				// Check the action type here...
				Action action = agent.getActionMap().get(CoreUtilities.createSignature(activity));
				if (action == null) {
					action = agent.getModuleLevelAction(activity);
					if (action == null) {
						// No valid action, so treat the activity as a term...
						// then quit the plan step
						planStack.updateBinding(variable, term);
						return;
					}
				}
					
				if (!ActiveAction.class.isInstance(action)) {
					System.err.println("[" + agent.getName() + "] Attempted to activate non-active action: " + CoreUtilities.presenter.toString(activity));
					planStack.updateBinding(variable, term);
					return;
				}
				
				List<Variable> variables = new LinkedList<Variable>();
				variables.add(variable);
				thread = new ActiveActionThread(activity, (ActiveAction) action, variables);
				if (durative) {
					new Thread(thread).start();
				} else {
					thread.run();
//					thread.finish(agent, planStack);
				}
			} else {
				planStack.updateBinding(variable, term);
				return;
			}
		}
		
		// each time the step is handled, check if the action has finished
		if (thread.isFinished()) {
			thread.finish(agent, planStack);
			thread = null;
		} else {
			// not finished, so put the step back on the plan stack.
			planStack.back();
		}			
	}
}
