package com.agentfactory.clf.plans;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.reasoner.Bindings;

public class DurativeStep implements IPlanStep {
	protected IFormula step;
	private ActionThread thread = null;
	
	public DurativeStep(IFormula step) {
		this.step = step;
	}
	
	@Override
	public IPlanStep apply(Bindings set) {
		return new DurativeStep(step.apply(set));
	}
	
	public IFormula formula() {
		return step;
	}

	public String toString() {
		return toString("");
	}
	
	public String toString(String tabs) {
		return tabs + "durative(" + CoreUtilities.presenter.toString(step) + ")";
	}

	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		// Lazy initialization of action thread (performed the first time
		// the step is handled
		if (thread == null) {
			IFormula activity = step.apply(planStack.bindings());
			if (Predicate.class.isInstance(activity)){
				thread = new ActionThread(agent, (Predicate) activity);
				new Thread(thread).start();
			}
		}
		
		// each time the step is handled, check if the action has finished
		if (thread.isFinished()) {
			thread.finish(planStack);
		} else {
			// not finished, so put the step back on the plan stack.
			planStack.back();
		}
	}
}
