package com.agentfactory.clf.plans;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.reasoner.Bindings;
import com.agentfactory.clf.reasoner.QueryResult;

public class IfStep implements IPlanStep {
	private IFormula guard;
	private IPlanStep ifStep;
	private IPlanStep elseStep;
	
	public IfStep(IFormula guard, IPlanStep ifStep) {
		this(guard, ifStep, null);
	}
	
	public IfStep(IFormula guard, IPlanStep ifStep, IPlanStep elseStep) {
		this.guard = guard;
		this.ifStep = ifStep;
		this.elseStep = elseStep;
	}
	
	@Override
	public IPlanStep apply(Bindings set) {
		if (elseStep == null) return new IfStep(guard.apply(set), ifStep.apply(set));
		return new IfStep(guard.apply(set), ifStep.apply(set), elseStep.apply(set));
	}

	public String toString() {
		return toString("");
	}
	
	public String toString(String tabs) {
		String output = tabs + "if (" + CoreUtilities.presenter.toString(guard) + ") ";
		if (!BlockStep.class.isInstance(ifStep)) {
			output += "\n" + ifStep.toString(tabs + IPlanStep.TAB);
		} else {
			output += ifStep.toString(tabs);
		}
		if (elseStep != null) {
			output += "\n" + tabs + "else ";
			if (!BlockStep.class.isInstance(elseStep)) {
				output += "\n" + elseStep.toString(tabs + IPlanStep.TAB);
			} else {
				output += elseStep.toString(tabs);
			}
		}
		return output;
	}

	public IFormula guard() {
		return guard;
	}
	
	public IPlanStep ifStep() {
		return ifStep;
	}

	public IPlanStep elseStep() {
		return elseStep;
	}
	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		QueryResult result = agent.queryAll(guard.apply(planStack.bindings()));
		List<IPlanStep> steps = new LinkedList<IPlanStep>();
		if (result.found) {
			if (result.bindings.isEmpty()) {
				steps.add(ifStep);
			} else {
				steps.add(ifStep.apply(result.bindings.get(0)));
			}
		} else {
			if (elseStep != null) {
				steps.add(elseStep);
			}
		}
		planStack.addPlan(steps);
	}
}
