package com.agentfactory.clf.plans;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.reasoner.Bindings;

public class TryRecoverStep implements IPlanStep {
	private IPlanStep plan;
	private IPlanStep recoverPlan;
	private int state;
	
	public TryRecoverStep(IPlanStep plan, IPlanStep recoverPlan) {
		this.plan = plan;
		this.recoverPlan = recoverPlan;
		state = 0;
	}
	
	@Override
	public IPlanStep apply(Bindings set) {
		return new TryRecoverStep(plan.apply(set), recoverPlan.apply(set));
	}

	public String toString() {
		return toString("");
	}
	
	public String toString(String tabs) {
		String output = tabs + "try ";
		if (!BlockStep.class.isInstance(plan)) {
			output += "\n" + plan.toString(tabs + IPlanStep.TAB) +"\n" + tabs;
		} else {
			output += plan.toString(tabs);
		}
		output += " recover ";
		if (!BlockStep.class.isInstance(recoverPlan)) {
			output += "\n" + recoverPlan.toString(tabs + IPlanStep.TAB);
		} else {
			output += recoverPlan.toString(tabs);
		}
		return output;
	}

	public IPlanStep plan() {
		return plan;
	}

	public IPlanStep recoverPlan() {
		return recoverPlan;
	}
	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		if (state == 0) {
			List<IPlanStep> steps = new LinkedList<IPlanStep>();
			steps.add(recoverPlan);
			planStack.addRecoveryPlan(steps);
			
			steps = new LinkedList<IPlanStep>();
			steps.add(plan);
			steps.add(this);
			planStack.addPlan(steps);
			state++;
		}
	}
}
