package com.agentfactory.clf.plans;

import com.agentfactory.clf.interpreter.AbstractLanguageAgent;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.reasoner.Bindings;

public class UpdateStep implements IPlanStep {
	protected IFormula step;
	protected String op;
	private Predicate selfSource = (Predicate) CoreUtilities.factory.createFormula("source(self)");
	
	public UpdateStep(String op, IFormula step) {
		this.op = op;
		this.step = step;
	}
	
	@Override
	public IPlanStep apply(Bindings set) {
		return new UpdateStep(op, step.apply(set));
	}
	
	public IFormula formula() {
		return step;
	}

	public String toString() {
		return toString("");
	}
	
	public String toString(String tabs) {
		return tabs + op + CoreUtilities.presenter.toString(step);
	}

	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		IFormula formula = step.apply(planStack.bindings());
		if (op.equals("+")) {
			adopt(agent, formula);
		} else if (op.equals("-")) {
			drop(agent, formula);
		}
	}
	
	protected void adopt(AbstractLanguageAgent agent, IFormula formula) {
		if (Predicate.class.isInstance(formula)) {
			Predicate predicate = (Predicate) formula.apply(new Bindings());
			if (predicate.getLabels().isEmpty()) {
				predicate.getLabels().add(selfSource);
			}
			agent.addBelief(predicate);
		}
	}
	
	protected void drop(AbstractLanguageAgent agent, IFormula formula) {
		if (Predicate.class.isInstance(formula)) {
			Predicate predicate = (Predicate) formula.apply(new Bindings());
			if (predicate.getLabels().isEmpty()) {
				predicate.getLabels().add(selfSource);
			}
			agent.removeBelief(predicate);
		}
	}
}
