package com.agentfactory.eis;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.agentfactory.clf.interpreter.Action;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.interpreter.Module;
import com.agentfactory.clf.interpreter.Sensor;
import com.agentfactory.clf.lang.Function;
import com.agentfactory.clf.lang.ITerm;
import com.agentfactory.clf.lang.Predicate;

import eis.exceptions.ManagementException;
import eis.iilang.Parameter;

public class EISManagerModule extends Module {
	EISService service;
	
	public EISManagerModule() {
		addAction("setup(?pid,?jarFile, ?params)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String pid = CoreUtilities.presenter.toString(activity.termAt(0));
				String jarFile = termAsString(activity, 1);
				List<ITerm> list = activity.termAt(2).terms();
				agent.bindToPlatformService(pid);
				service = (EISService) agent.getPlatformService(pid);
				service.setup(jarFile);
				
				if (service.getEnvironmentState().equals("initializing")) {
					Map<String, Parameter> parameters = new HashMap<String, Parameter>();
					for (ITerm term : list) {
						Function f = (Function) term;
						parameters.put(f.functor(), Utilities.toParameter(f.termAt(0)));
					}
					try {
						service.eisInit(parameters);
					} catch (ManagementException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				return true;
			}
		});
		addAction("getFreeEntities", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				for (String entity : service.getFreeEntities()) {
					addBelief("freeEntity(" + entity + ")");
				}
				return true;
			}
		});
		addAction("getEntityType(?entity)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String entity = CoreUtilities.presenter.toString(activity.termAt(0));
				String type = service.queryEntityType(entity);
				addBelief("entityType(" + entity + "," + type + ")");
				return true;
			}
		});
		addAction("start", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				try {
					service.eisStart();
				} catch (ManagementException e) {
					System.out.println("Failed to start EIS Environment...");
					return false;
				}
				return true;
			}
		});
		addSensor("eisSensor", new Sensor() {
			@Override
			public void perceive() {
				if (service == null) {
					addBelief("eisState(disconnected)");
					return;
				}
				
				addBelief("eisState(" + service.getEnvironmentState() + ")");

				if (service.isStartSupported()) {
					addBelief("eisSupports(start)");
				}
				
				if (service.hasEnvironmentEvent()) {
					addBelief(service.getNextEnvironmentEvent());
				}
			}
		});
	}
}
