package com.agentfactory.eis;

import java.util.Map;

import com.agentfactory.afapl2.interpreter.AFAPL2ArchitectureFactory;
import com.agentfactory.agentspeak.interpreter.AgentSpeakArchitectureFactory;
import com.agentfactory.platform.core.ConfigurationException;
import com.agentfactory.platform.impl.DefaultRunConfiguration;
import com.agentfactory.teleoreactive.interpreter.TeleoReactiveArchitectureFactory;

public class EISRunConfiguration extends DefaultRunConfiguration {
	private String jar, name;
	Map<String, String> designs;
	
	public EISRunConfiguration(String name, Map<String, String> designs, String jar) {
		this.name = name;
		this.jar = jar;
		this.designs = designs;
	}
	
	public String getName() {
		return name;
	}
	
	public String getDomain() {
		return "eis.ucd.ie";
	}
	
	public void configure() throws ConfigurationException {
		super.configure();
		
		addArchitectureFactory(new AFAPL2ArchitectureFactory());
		addArchitectureFactory(new AgentSpeakArchitectureFactory());
		addArchitectureFactory(new TeleoReactiveArchitectureFactory());
	
		addPlatformService(EISService.class, "eis");
		
        addAgent("manager", "com/agentfactory/eis/EnvironmentManager.aspeak");
        initAgent("manager", "environment(eis, \"" + jar + "\")");
        for (String design: designs.keySet()) {
        	initAgent("manager", "design("+design+", \"" + designs.get(design) + "\")");
        }
        resumeAgent("manager");
	}
}
