package com.agentfactory.eis;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Collection;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.impl.AbstractPlatformService;

import eis.AgentListener;
import eis.EILoader;
import eis.EnvironmentInterfaceStandard;
import eis.EnvironmentListener;
import eis.exceptions.ActException;
import eis.exceptions.AgentException;
import eis.exceptions.EntityException;
import eis.exceptions.ManagementException;
import eis.exceptions.NoEnvironmentException;
import eis.exceptions.PerceiveException;
import eis.exceptions.RelationException;
import eis.iilang.Action;
import eis.iilang.EnvironmentState;
import eis.iilang.Parameter;
import eis.iilang.Percept;


public class EISService extends AbstractPlatformService {
	private EnvironmentInterfaceStandard ei = null;
	private Map<String, List<Percept>> events = new Hashtable<String, List<Percept>>();
	private List<String> environmentEventQueue = new LinkedList<String>();
	
	private class EISAgentListener implements AgentListener {
		@Override
		public void handlePercept(String agentName, Percept percept) {
			events.get(agentName).add(percept);
		}
	}
	
	private class EISEnvironmentListener implements EnvironmentListener {
		@Override
		public void handleNewEntity(String arg0) {
			environmentEventQueue.add("newEntity(" + arg0 + ")");
		}

		@Override
		public void handleDeletedEntity(String entity, Collection<String> agents) {
			environmentEventQueue.add("deletedEntity(" + entity+ ")");
		}

		@Override
		public void handleFreeEntity(String entity, Collection<String> agents) {
			environmentEventQueue.add("freedEntity(" + entity+ ")");
		}

		@Override
		public void handleStateChange(EnvironmentState newState) {
			// TODO Auto-generated method stub
			
		}
	}

	public void eisStart() throws ManagementException {
		ei.start();
	}
	
	public void eisInit(Map<String, Parameter> parameters) throws ManagementException {
		ei.init(parameters);
	}
	
	public boolean isStartSupported() {
		return ei.isStartSupported();
	}
	
	public String getEnvironmentState() {
		EnvironmentState state = ei.getState();
		if (state.equals(EnvironmentState.PAUSED)) {
			return "paused";
		} else if (state.equals(EnvironmentState.INITIALIZING)) {
			return "initializing";
		} else if (state.equals(EnvironmentState.RUNNING)) {
			return "running";
		} else if (state.equals(EnvironmentState.KILLED)) {
			return "killed";
		}
		return "unknown";
	}
	
	public String getNextEnvironmentEvent() {
		return environmentEventQueue.remove(0);
	}
	
	public boolean hasEnvironmentEvent() {
		return !environmentEventQueue.isEmpty();
	}
	
    public void setup(String jarFile) {
		try {
			if (jarFile.startsWith("jar:")) {
				System.out.println("[UNSUPPORTED] Using URL-based download: " + jarFile);
//				ei = EILoader.fromJarFile(new URL(jarFile));
			} else {
				ei = EILoader.fromJarFile(new File(jarFile));
			}
			EISEnvironmentListener listener = new EISEnvironmentListener();
			ei.attachEnvironmentListener(listener);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    
    public void registerAgent(IAgent agent) {
    	try {
			ei.registerAgent(agent.getName());
			events.put(agent.getName(), new LinkedList<Percept>());
			ei.attachAgentListener(agent.getName(), new EISAgentListener());
		} catch (AgentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    
    public void unregisterAgent(IAgent agent) {
    	try {
			ei.unregisterAgent(agent.getName());
		} catch (AgentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    
    public Collection<String> getFreeEntities() {
    	return ei.getFreeEntities();
    }
    
    public boolean associateEntity(IAgent agent, String entity) {
    	try {
			ei.associateEntity(agent.getName(), entity);
		} catch (RelationException e) {
			// TODO Auto-generated catch block
			return false;
		}
		return true;
    }

    public String queryEntityType(String entity) {
    	try {
			return ei.getType(entity);
		} catch (EntityException e) {
		}
		return null;
    }

    public Map<String, Percept> performAction(IAgent agent, Action act) throws ActException, NoEnvironmentException {
//    	System.out.println(act.toXML());
		return ei.performAction(agent.getName(), act);
		
    }
    
    public Map<String, Collection<Percept>> getPercepts(IAgent agent) throws PerceiveException, NoEnvironmentException {
		return ei.getAllPercepts(agent.getName(), agent.getName());
    }
    
    public List<Percept> getAgentEvents(IAgent agent) {
    	return events.get(agent.getName());
    }
    
}
