/*									tab:4
 * Copyright (c) 2007 University College Dublin.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL UNIVERSITY COLLEGE DUBLIN BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF 
 * UNIVERSITY COLLEGE DUBLIN HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * UNIVERSITY COLLEGE DUBLIN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND UNIVERSITY COLLEGE DUBLIN HAS NO
 * OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 *
 *
 */
package com.agentfactory.eis.afapl2;

import java.util.LinkedList;
import java.util.Map;

import com.agentfactory.logic.agent.Actuator;
import com.agentfactory.logic.lang.FOS;

import eis.exceptions.ActException;
import eis.exceptions.NoEnvironmentException;
import eis.iilang.Identifier;
import eis.iilang.Numeral;
import eis.iilang.Parameter;
import eis.iilang.Percept;

/**
 * 
 * @author Mauro Dragone
 */
public class Perform extends Actuator {
	@Override
	public boolean act(FOS action) {
		EISControlModule eis = (EISControlModule) getModuleByName("eis");
		if (eis == null) {
			adoptBelief("BELIEF(noSuchModule(eis))");
			return false;
		}
		FOS arg = action.argAt(0);
		String act = arg.getFunctor();
		
		LinkedList<Parameter> list = new LinkedList<Parameter>();
		for (int i=0; i<arg.numArguments(); i++) {
			String term = arg.argAt(i).toString();
			try {
				list.add(new Numeral(Double.parseDouble(term)));
			} catch (Throwable th) {
				try {
					list.add(new Numeral(Integer.parseInt(term)));
				} catch (Throwable th2) {
					list.add(new Identifier(term));
				}
			}
		}
		
		try {
			Map<String, Percept> percepts = eis.service.performAction(agent, new eis.iilang.Action(act, list));

			for (Percept percept: percepts.values()) {
				adoptBelief(eis.generateBelief(percept));
			}
			return true;
		} catch (ActException e) {
			if( e.getType() == ActException.NOTSPECIFIC) {
				System.out.println(agent.getName() + ": Action Failure (NOTSPECIFIC): " + action);
				} else if( e.getType() == ActException.NOTREGISTERED ) {
					System.out.println(agent.getName() + ": Action Failure (NOTREGISTERED): " + action);
				} else if( e.getType() == ActException.NOENTITIES ) {
					System.out.println(agent.getName() + ": Action Failure (NOENTITIES): " + action);
				} else if( e.getType() == ActException.WRONGENTITY ) {
					System.out.println(agent.getName() + ": Action Failure (WRONGENTITY): " + action);
				} else if( e.getType() == ActException.NOTSUPPORTEDBYENTITY ) {
					System.out.println(agent.getName() + ": Action Failure (NOTSUPPORTEDBYENTITY): " + action);
				} else if( e.getType() == ActException.NOTSUPPORTEDBYENVIRONMENT ) {
					System.out.println(agent.getName() + ": Action Failure (NOTSUPPORTEDBYENVIRONMENT): " + action);
				} else if( e.getType() == ActException.NOTSUPPORTEDBYTYPE ) {
					System.out.println(agent.getName() + ": Action Failure (NOTSUPPORTEDBYTYPE): " + action);
				} else if( e.getType() == ActException.FAILURE ) {
					System.out.println(agent.getName() + ": Action Failure (FAILURE): " + action);
				}
			adoptBelief("failedAction(" + action + ")");
		} catch (NoEnvironmentException e) {
			System.out.println(agent.getName() + ": Action Failed: " + action);
			adoptBelief("failedAction(" + action + ")");
		}
		return false;
	}
}
