/**
 * Title:       Module.java
 * Copyright:   Copyright (c) 1996-2004 The Agent Factory Working Group. All rights reserved.
 * Licence:     This file is free software; you can redistribute it and/or modify
 *              it under the terms of the GNU Lesser General Public License as published by
 *              the Free Software Foundation; either version 2.1, or (at your option)
 *              any later version.
 *
 *              This file is distributed in the hope that it will be useful,
 *              but WITHOUT ANY WARRANTY; without even the implied warranty of
 *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *              GNU Lesser General Public License for more details.
 *
 *              You should have received a copy of the GNU Lesser General Public License
 *              along with Agent Factory; see the file COPYING.  If not, write to
 *              the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *              Boston, MA 02111-1307, USA.
 */
package com.agentfactory.logic.agent;

import com.agentfactory.logic.MentalAgent;
import java.util.Map;


/**
 * This is an abstract base class from which all agent modules should be
 * subclassed.  Whenever a module is created for an agent, a reference to
 * that agent is passed into the module, allowing code to reference the agent.
 *
 * @author  Rem Collier
 */
public abstract class Module {
    private String name;
    
    /**
     * A reference to the parent agent.
     */
    protected MentalAgent agent;
    
    protected Map<String,String> configuration;
    
    /**
     * Factory Style constructor
     */
    public void init() {
    }
    
    /**
     * Set the agent.
     */
    public void setAgent(MentalAgent agent) {
        this.agent = agent;
    }
    
    public String toConstructString(){
        return new String("DEFAULT");
    }
    
    public void fromConstructString(String constructString){
    }
    
    public void setName(String name) {
        this.name = name;
    }
    
    public String getName() {
        return name;
    }
    
    public void destroy(){
    }
    
    public void setConfiguration(Map<String,String> configuration) {
        this.configuration = configuration;
    }
    
    public Map<String,String> getConfiguration() {
        return configuration;
    }
}
