package com.agentfactory.logic.agent.beliefManagement;

import com.agentfactory.logic.agent.beliefManagement.BeliefStore;
import com.agentfactory.logic.agent.beliefManagement.IBeliefManager;
import com.agentfactory.logic.lang.FOS;
import java.util.LinkedList;
import java.util.List;

/**
 * A basic belief manager provides support for the storage and querying of
 * a set of atomic beliefs (i.e. facts).
 *
 * @author Rem Collier
 */
public class BasicBeliefManager implements IBeliefManager {
    protected BeliefStore beliefs;

    public BasicBeliefManager() {
        beliefs = new BeliefStore("current");
    }


    public void adopt(FOS belief) {
        if (belief.getFunctor().equals("BELIEF")) {
            beliefs.addBelief(belief);
        }
    }

    public void drop(FOS belief) {
        if (belief.getFunctor().equals("BELIEF")) {
            beliefs.removeBelief(belief);
        }
    }

    public void update() {
        // THIS METHOD DOES NOTHING
    }

    public boolean hasType(String type) {
        return type.equals("BELIEF");
    }

    public List<Object> query(FOS query) {
        List<Object> list = new LinkedList<Object>();
        list.addAll(beliefs.query(query));
        return list;
    }

    public List<FOS> getBeliefs() {
        return beliefs.getBeliefs();
    }
}
