/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.agentfactory.logic.agent.beliefManagement;

import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.reasoner.Queryable;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 *
 * @author rem
 */
public class BeliefStore implements Queryable {
    private String label;
    private List<FOS> beliefs;
    private Map<String, List<FOS>> beliefMap;

    public BeliefStore(String label) {
        this.label = label;
        beliefs = new LinkedList<FOS>();
        beliefMap = new HashMap<String, List<FOS>>();
    }

    public String getLabel() {
        return label;
    }

    public void clear() {
        beliefs.clear();
        beliefMap.clear();
    }

    public void addBelief(FOS belief) {
        beliefs.add(belief);
        List<FOS> list = beliefMap.get(belief.getFunctor());
        if (list == null) {
            list = new LinkedList<FOS>();
            beliefMap.put(belief.getFunctor(), list);
        }
        list.add(belief);
    }

    public void removeBelief(FOS belief) {
        beliefs.remove(belief);
        List<FOS> list = beliefMap.get(belief.getFunctor());
        if (list != null) {
            list.remove(belief);
        }
    }

    public void refresh(List<FOS> beliefList) {
        clear();
        for (FOS belief: beliefList) {
            addBelief(belief);
        }
    }

    /**
     * A belief store only works with BELIEF type FOSs
     * 
     * @param type
     * @return
     */
    public boolean hasType(String type) {
        return type.equals("BELIEF");
    }

    /**
     * Generate a list of matching beliefs
     */
    public List<Object> query(FOS query) {
        List<Object> list = new LinkedList<Object>();
        List<FOS> options = beliefMap.get(query.getFunctor());
        if (options != null) {
            for(FOS belief: options) {
                if (query.matches(belief)) {
                    list.add(belief);
                }
            }
        }
        return list;
    }

    public List<FOS> getBeliefs() {
        return beliefs;
    }

}
