package com.agentfactory.logic.agent.beliefManagement;

import com.agentfactory.logic.MentalAgent;
import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.lang.TerImplication;
import com.agentfactory.logic.reasoner.QueryEngine;
import java.util.LinkedList;
import java.util.List;

/**
 * This belief manager implements the traditional Agent Factory belief
 * management model. It incorporates perception based belief (via the
 * PerceptionBasedBeliefManager) class; basic and temporal beliefs (via
 * the TemporalBeliefStore); and belief rules (via the BeliefRuleStore).
 * beliefs
 * @author rem
 */
public class TemporalBeliefManager extends PerceptionBasedBeliefManager implements IBeliefManagerRuleSupport {
    protected TemporalBeliefStore temporalBeliefStore;
    protected List<FOS> incomingBeliefs;
    protected BeliefRuleStore beliefRules;

    public TemporalBeliefManager(MentalAgent agent, QueryEngine queryEngine) {
        super(agent);

        incomingBeliefs = new LinkedList<FOS>();
        temporalBeliefStore = new TemporalBeliefStore(queryEngine);
        beliefRules = new BeliefRuleStore();
    }

    @Override
    public void adopt(FOS belief) {
        incomingBeliefs.add(belief);
    }

    @Override
    public void drop(FOS belief) {
        temporalBeliefStore.removeBelief(belief);
    }

    @Override
    public void update() {
        // start updating the temporal beliefs (clear basic beliefs and
        // update NEXT and ALWAYS temporal beliefs)
        temporalBeliefStore.updateNEXTAndALWAYSBeliefs();

        // Insert new beliefs
        for (FOS belief: incomingBeliefs) {
            temporalBeliefStore.addBelief(belief);
        }

        incomingBeliefs.clear();

        // Invoke perception process
        super.update();

        // Update UNTIL beliefs
        temporalBeliefStore.updateUNTILBeliefs();
    }

    @Override
    public List<Object> query(FOS query) {
        List<Object> list = super.query(query);
        list.addAll(temporalBeliefStore.query(query));
        list.addAll(beliefRules.query(query));
        return list;
    }

    @Override
    public List<FOS> getBeliefs() {
        List<FOS> list = new LinkedList<FOS>();
        list.addAll(temporalBeliefStore.getBeliefs());
        for (BeliefStore store:perceptions.values()) {
            list.addAll(store.getBeliefs());
        }
        return list;
    }

    public boolean hasType(String type) {
        return type.equals("BELIEF");
    }

    public void addRule(TerImplication rule) {
        beliefRules.addRule(rule);
    }

    public List<TerImplication> getRules() {
        return beliefRules.getRules();
    }

    public List<FOS> getTemporalBeliefs() {
        return temporalBeliefStore.getTemporalBeliefs();
    }
}
