/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.agentfactory.logic.agent.beliefManagement;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.logic.lang.FOS;
import com.agentfactory.logic.reasoner.QueryEngine;

/**
 *
 * @author rem
 */
public class TemporalBeliefStore extends BeliefStore {
    private List<FOS> temporalBeliefs;
    private QueryEngine queryEngine;

    public TemporalBeliefStore(QueryEngine queryEngine) {
        super("temporal");
        this.queryEngine = queryEngine;
        temporalBeliefs = new LinkedList<FOS>();
    }

    @Override
    public void addBelief(FOS belief) {
        if (isTemporalBelief(belief)) {
            temporalBeliefs.add(belief);
            if (belief.getFunctor().equals("ALWAYS")) {
                super.addBelief(belief.argAt(0));
            } else if (belief.getFunctor().equals("UNTIL")) {
                super.addBelief(belief.argAt(1));
            }
        } else {
            super.addBelief(belief);
        }
    }

    @Override
    public void clear() {
        super.clear();
        temporalBeliefs = new LinkedList<FOS>();
    }

    @Override
    public void removeBelief(FOS belief) {
        if (isTemporalBelief(belief)) {
            temporalBeliefs.remove(belief);
            if (belief.getFunctor().equals("ALWAYS")) {
                super.removeBelief(belief.argAt(0));
            } else if (belief.getFunctor().equals("UNTIL")) {
                super.removeBelief(belief.argAt(1));
            }
        } else {
            super.removeBelief(belief);
        }
    }

    private boolean isTemporalBelief(FOS belief) {
        if (belief.getFunctor().equals("NEXT")) return true;
        if (belief.getFunctor().equals("ALWAYS")) return true;
        if (belief.getFunctor().equals("UNTIL")) return true;
        return false;
    }

    public void updateNEXTAndALWAYSBeliefs() {
        // then create a new temporal belief set.
        List<FOS> oldBeliefs = temporalBeliefs;
        clear();
        
        for (FOS belief: oldBeliefs) {
            if (belief.getFunctor().equals("NEXT")) {
                addBelief(belief.argAt(0));
            } else if (belief.getFunctor().equals("ALWAYS")) {
                addBelief(belief);
            } else if (belief.getFunctor().equals("UNTIL")) {
                temporalBeliefs.add(belief);
            }
        }
    }

    /**
     * Performs temporal update of UNTIL beliefs held by the agent.
     */
    public void updateUNTILBeliefs() {
        List<FOS> oldBeliefs = temporalBeliefs;
        temporalBeliefs = new LinkedList<FOS>();

        for (FOS belief: oldBeliefs) {
            if (belief.getFunctor().equals("NEXT")) {
                // Add the contained belief to the beliefs
                temporalBeliefs.add(belief);
            } else if (belief.getFunctor().equals("ALWAYS")) {
                // Add the belief again
                temporalBeliefs.add(belief);
            } else if (belief.getFunctor().equals("UNTIL")) {
                if (!queryEngine.query(belief.argAt(0))) {
                    addBelief(belief);
                }
            }
        }
    }

    public List<FOS> getTemporalBeliefs() {
        return temporalBeliefs;
    }

}
