package com.agentfactory.logic.update.lang;

import java.util.HashSet;
import java.util.Set;

import com.agentfactory.logic.update.interpreter.CoreUtilities;
import com.agentfactory.logic.update.reasoner.Bindings;

public class Division implements ITerm {
	private ITerm[] terms;
	
	public Division(ITerm[] terms) {
		this.terms = terms;
	}
	
	@Override
	public ITerm apply(Bindings set) {
		ITerm[] t = new ITerm[2];
		t[0] = terms[0].apply(set);
		t[1] = terms[1].apply(set);
		if (t[0].isConstant() && t[1].isConstant()) {
			if (IntegerTerm.class.isInstance(t[0])) {
				if (IntegerTerm.class.isInstance(t[1])) {
					int answer = ((IntegerTerm) t[0]).value() / ((IntegerTerm) t[1]).value();
					return new IntegerTerm(answer);
				} else if (RealTerm.class.isInstance(t[1])) {
					double answer = ((IntegerTerm) t[0]).value() / ((RealTerm) t[1]).value();
					return new RealTerm(answer);
				} else {
					throw new NumberFormatException();
				}
			} else if (RealTerm.class.isInstance(t[0])) {
				if (IntegerTerm.class.isInstance(t[1])) {
					double answer = ((RealTerm) t[0]).value() / ((IntegerTerm) t[1]).value();
					return new RealTerm(answer);
				} else if (RealTerm.class.isInstance(t[1])) {
					double answer = ((RealTerm) t[0]).value() / ((RealTerm) t[1]).value();
					return new RealTerm(answer);
				} else {
					throw new NumberFormatException();
				}
			} else {
				throw new NumberFormatException();
			}
		}
		return new Division(t);
	}

	@Override
	public boolean containsVariable(Variable variable) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isCompatibleFunction(ITerm term) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isConstant() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isDistinctFunction(ITerm term) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isNotFunctional(ITerm term) {
		// TODO Auto-generated method stub
		return false;
	}

	public ITerm term(int i) {
		return terms[i];
	}

	@Override
	public boolean hasTerms() {
		// Ignore this property for arithmetic
		// TODO (Rem Collier): Refactor this to be better...
		return false;
	}
	
	@Override
	public Set<Variable> getVariables() {
		Set<Variable> set= new HashSet<Variable>();
		set.addAll(terms[0].getVariables());
		set.addAll(terms[1].getVariables());
		return set;
	}

	public boolean matches(ITerm term) {
		if (!Division.class.isInstance(term)) {
			return false;
		}
		
		Division op = (Division) term;
		return terms[0].matches(op.terms[0]) && terms[1].matches(op.terms[1]);
	}
}
