package com.agentfactory.logic.update.plans;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.logic.update.interpreter.AbstractLanguageAgent;
import com.agentfactory.logic.update.reasoner.Bindings;

public class BlockStep implements IPlanStep {
	private List<IPlanStep> plan;
	
	public BlockStep(List<IPlanStep> plan) {
		this.plan = plan;
	}
	
	@Override
	public IPlanStep apply(Bindings set) {
		List<IPlanStep> list = new LinkedList<IPlanStep>();
		for (IPlanStep step: plan) {
			list.add(step.apply(set));
		}
		return new BlockStep(list);
	}

	public String toString() {
		String output = "{";
		boolean first = true;
		for (IPlanStep step: plan) {
			if (first) first = false;
			else output += ",";
			output += step.toString();
		}
		return output + "}";
	}

	public List<IPlanStep> plan() {
		return plan;
	}

	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		planStack.pushBindings();
		List<IPlanStep> steps = new LinkedList<IPlanStep>();
		for (IPlanStep stp: plan) {
			steps.add(stp);
		}
		planStack.addPlan(steps);
	}
}
