package com.agentfactory.logic.update.plans;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.logic.update.interpreter.AbstractLanguageAgent;
import com.agentfactory.logic.update.interpreter.CoreUtilities;
import com.agentfactory.logic.update.lang.IFormula;
import com.agentfactory.logic.update.reasoner.Bindings;
import com.agentfactory.logic.update.reasoner.QueryResult;

public class IfStep implements IPlanStep {
	private IFormula guard;
	private List<IPlanStep> ifPlan;
	private List<IPlanStep> elsePlan;
	
	public IfStep(IFormula guard, List<IPlanStep> ifPlan, List<IPlanStep> elsePlan) {
		this.guard = guard;
		this.ifPlan = ifPlan;
		this.elsePlan = elsePlan;
	}
	@Override
	public IPlanStep apply(Bindings set) {
		List<IPlanStep> ifList = new LinkedList<IPlanStep>();
		for (IPlanStep step: ifPlan) {
			ifList.add(step.apply(set));
		}
		List<IPlanStep> elseList = new LinkedList<IPlanStep>();
		for (IPlanStep step: elsePlan) {
			elseList.add(step.apply(set));
		}
		return new IfStep(guard.apply(set), ifList, elseList);
	}

	public String toString() {
		String output = "if (" + CoreUtilities.presenter.toString(guard) + ") {";
		boolean first = true;
		for (IPlanStep step: ifPlan) {
			if (first) first = false;
			else output += ",";
			output += step.toString();
		}
		output += "} else {";
		first = true;
		for (IPlanStep step: elsePlan) {
			if (first) first = false;
			else output += ",";
			output += step.toString();
		}
		return output + "}";
	}

	public IFormula guard() {
		return guard;
	}
	
	public List<IPlanStep> ifPlan() {
		return ifPlan;
	}

	public List<IPlanStep> elsePlan() {
		return elsePlan;
	}
	@Override
	public void handle(AbstractLanguageAgent agent, PlanStack planStack) {
		QueryResult result = agent.queryAll(guard.apply(planStack.bindings()));
		List<IPlanStep> steps = new LinkedList<IPlanStep>();
		if (result.found) {
			if (result.bindings.isEmpty()) {
				for (IPlanStep stp: ifPlan) {
					steps.add(stp);
				}
			} else {
				for (IPlanStep stp: ifPlan) {
					steps.add(stp.apply(result.bindings.get(0)));
				}
			}
		} else {
			for (IPlanStep stp: elsePlan) {
				steps.add(stp);
			}
		}
		planStack.addPlan(steps);
	}
}
