package com.agentfactory.platform.impl;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import com.agentfactory.platform.core.ConfigurationException;
import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.core.IAgentPlatform;
import com.agentfactory.platform.core.IArchitectureFactory;
import com.agentfactory.platform.core.IPlatformService;
import com.agentfactory.platform.core.IRunConfiguration;
import com.agentfactory.platform.core.NoSuchServiceException;
import com.agentfactory.service.ams.AgentManagementService;

public class DefaultRunConfiguration implements IRunConfiguration {
	DefaultAgentPlatform platform;
	AbstractPlatformServiceManager manager;
	Map<String, IAgent> agents;
	AgentManagementService ams;
	
	public DefaultRunConfiguration() {
		agents = new HashMap<String, IAgent>();
	}
	
	public void configure() {
    	platform = new DefaultAgentPlatform();
        platform.setName(getName());
        platform.setDomain(getDomain());
//        platform.setScheduler(new com.agentfactory.platform.impl.RoundRobinTimeSliceScheduler());
        platform.setScheduler(new com.agentfactory.platform.impl.MultiThreadFixedScheduler());
        manager = (AbstractPlatformServiceManager) platform.getPlatformServiceManager();
        ams = (AgentManagementService) manager.getServiceByName(AgentManagementService.NAME);
	}
	
	protected void addArchitectureFactory(IArchitectureFactory factory) {
        platform.getArchitectureService().registerArchitectureFactory(factory);
	}

	public void addPlatformService(Class<? extends IPlatformService> serviceClass, String name) throws ConfigurationException { 
		try {
	        manager.addService(serviceClass, name);
		} catch (NoSuchServiceException e) {
			throw new ConfigurationException(e);
		}
	}
	
	protected void addPlatformService(Class<? extends IPlatformService> serviceClass, String name, Properties props) throws ConfigurationException { 
		try {
	        manager.addService(serviceClass, name, 0, props);
		} catch (NoSuchServiceException e) {
			throw new ConfigurationException(e);
		}
	}
	
	protected void addAgent(String name, String design) throws ConfigurationException {
		try {
	    	IAgent agent = ams.createAgent(name, design);
	    	agents.put(name, agent);
		} catch (Exception e) {
			throw new ConfigurationException(e); 
		}
	}
	
	protected void initAgent(String name, String init) {
		IAgent agent = agents.get(name);
		if (agent == null) System.out.println("WARNING: No agent with name: " + name);
		else agent.initialise(init);
	}
	
	protected void resumeAgent(String name) {
        ams.resumeAgent(name);
    }
	
	protected String getName() {
		return "default";
	}
	
	protected String getDomain() {
		return "some.domain.com";
	}

	@Override
	public IAgentPlatform getAgentPlatform() {
		return platform;
	}
}
