package com.agentfactory.platform.impl;

import com.agentfactory.platform.core.AgentID;
import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.core.IMessage;

/**
 * This abstract class outlines the methods that should be implemented by any
 * message transport service.
 *
 * @author  Rem Collier
 */
public abstract class MessageTransportService extends AbstractPlatformService {

    public abstract boolean isValidService( String address );

    public abstract boolean sendMessage( String address, IMessage message );

    public abstract String getAddress();

    public abstract String getServiceID();

    @Override
    public void onBind( IAgent agent ) {
        addObserver( agent );
    }

    @Override
    public void onUnbind( IAgent agent ) {
        deleteObserver( agent );
    }

    /**
     * This method is invoked from within a specific MTS implementation, and
     * is used to pass a received message on to the relevant agent.
     * 
     * @param message
     * @return
     */
    public synchronized boolean receiveMessage( IMessage message ) {
        AgentID aid = (AgentID) message.getReceivers().get( 0 );
        // '*' is a special case to match all agents
        // It's not an allowable regexp in any case, so it would cause an error otherwise
        boolean all = aid.getName().equals( "*" );
        for ( IAgent agent : getBoundAgents() ) {
            if ( all || agent.getName().matches( aid.getName() ) ) {
                agent.receive( message );
            }
        }
        
        return true;
    }
}

