package com.agentfactory.service.ams;

import java.util.ArrayList;
import java.util.List;

import com.agentfactory.platform.core.AgentCreationException;
import com.agentfactory.platform.core.DuplicateAgentNameException;
import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.core.IAgentPlatform;
import com.agentfactory.platform.core.NoSuchAgentException;
import com.agentfactory.platform.core.NoSuchArchitectureException;
import com.agentfactory.platform.impl.AbstractPlatformService;

/**
 * Agent Management Service
 *
 * @author Rem Collier
 * @author daithi
 */
public class AgentManagementService extends AbstractPlatformService {
    public static final String NAME = "fipa.std.service.ams";

    public AgentManagementService() {
        setName(NAME);
    }
    
    public synchronized List<String> getAgentNames() {
        List<String> list = new ArrayList<String>();

        IAgentPlatform platform = manager.getAgentPlatform();
        for ( IAgent agent:platform.getAgentContainer().agents() ) {
            list.add( agent.getName() );
        }

        return list;
    }

    public synchronized boolean resumeAgent( String name ) {
        manager.getAgentPlatform().getScheduler().resume(name);
        setChanged();
        notifyObservers("resumed " + name);
        return true;
    }

    public synchronized boolean suspendAgent( String name ) {
        manager.getAgentPlatform().getScheduler().suspend(name);
        setChanged();
        notifyObservers("suspended " + name);
        return true;
    }

    public synchronized boolean terminateAgent( String name ) {
        try {
        	// Setting the agent to the terminated state will cause it to to be dropped by the scheduler
            IAgent agent = manager.getAgentPlatform().getAgentContainer().getAgentByName( name );
            agent.setState( IAgent.TERMINATED );

            setChanged();
            notifyObservers("terminated " + agent.getName());
        }
        catch ( NoSuchAgentException e ) {
            return false;
        }
        return true;
    }

    /**
     * Create an agent based on an agent description file.
     *
     * Currently, agents can be created from .afapl code files. .agt is no
     * longer directly supported.
     *
     * @param inName -
     *            A String Object. The name of the agent that is to be
     *            constructed.sout
     * @param inType -
     *            A String Object, the name of the code file that is to be used
     *            to construct the agent.
     * @throws NoSuchArchitectureException -
     *             thrown if there is a problem constructing an agent.
     * @throws AgentCreationException 
     */
    public synchronized IAgent createAgent( String name, Object design ) throws NoSuchArchitectureException, DuplicateAgentNameException, AgentCreationException {
        IAgentPlatform platform = manager.getAgentPlatform();
        IAgent agent = platform.getArchitectureService().createAgent( name, design );
        platform.getAgentContainer().add(agent);
        platform.getScheduler().register( agent );

        setChanged();
        notifyObservers("created " + agent.getName());
        
        return agent;
    }
    
    public AgentDetails getAgentDetails(String name) {
		try {
			IAgent agent = manager.getAgentPlatform().getAgentContainer().getAgentByName(name);
			if (agent == null) return null;
	    	return new AgentDetails(name, agent.getType(), agent.getState(), agent.getAgentID());
		} catch (NoSuchAgentException e) {
			e.printStackTrace();
			return null;
		}
    }
}
