package com.agentfactory.service.df;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.platform.core.AgentID;
import com.agentfactory.platform.impl.AbstractPlatformService;

public class DirectoryFacilitatorService extends AbstractPlatformService {
    public static final String NAME = "fipa.df";
    private List<DFAgentDescription> descriptions;
    
    public DirectoryFacilitatorService() {
    	setName(NAME);
    	descriptions = new LinkedList<DFAgentDescription>();    	
    }
    
    /**
     * Register a new Agent Description with the DFS
     *  
     * @param description
     */
    public void register(DFAgentDescription description) {
    	for (DFAgentDescription desc: descriptions) {
    		if (desc.agentID.equals(description.agentID)) {
    			desc.merge(description);
    			return;
    		}
    	}
    	descriptions.add(description);
    }

    /**
     * Deregister an existing Agent Description with the DFS
     * 
     * @param agentID
     */
    public void deregister(AgentID agentID) {
    	int index = 0;
    	while (index < descriptions.size()) {
    		if (descriptions.get(index).agentID.equals(agentID)) {
    			descriptions.remove(index);
    			return;
    		}
    		index++;
    	}
    }
    
    public void modify(DFAgentDescription description) {
    	int index = 0;
    	while (index < descriptions.size()) {
    		if (descriptions.get(index).agentID.equals(description.agentID)) {
    			descriptions.set(index, description);
    			return;
    		}
    		index++;
    	}
    }

    public List<DFAgentDescription> search(DFAgentDescription description) {
    	List<DFAgentDescription> list = new LinkedList<DFAgentDescription>();
    	for (DFAgentDescription desc : descriptions) {
    		if (desc.matches(description)) {
    			list.add(desc);
    		}
    	}
    	return list;
    }
    
    public List<DFAgentDescription> get(AgentID agentID) {
    	List<DFAgentDescription> list = new LinkedList<DFAgentDescription>();
    	for (DFAgentDescription desc : descriptions) {
    		if (desc.agentID.matches(agentID)) {
    			list.add(desc);
    		}
    	}
    	return list;
    }
}
