package com.agentfactory.service.migration;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.HashMap;
import java.util.Map;

import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.impl.AbstractPlatformService;
import com.agentfactory.service.ams.AgentManagementService;

public class MigrationService extends AbstractPlatformService {
	public static final String NAME								= "af.std.mig";
	
	AgentManagementService ams;
	MigrationServer server;
	Thread thread;
	int port;
	Map<String, IMigrationHandler> handlers = new HashMap<String, IMigrationHandler>();
	
	public void addMigrationHandler(IMigrationHandler handler) {
		handlers.put(handler.type(), handler);
	}
	
	public boolean sendAgent(IAgent agent, String host, int port) {
		IMigrationHandler handler = handlers.get(agent.getType());
		if (handler == null) {
			System.out.println( "[Migration Service] Migration Failure: No Handler for agent: " + agent.getName() +
								" of type: " + agent.getType());
			return false;
		}
		
		IMigrationSnapshot snapshot = handler.create(agent);
		
		Socket socket = null;
		ObjectOutputStream out = null;
		ObjectInputStream in = null;
		try {
			socket = new Socket(host, port);
			out = new ObjectOutputStream(socket.getOutputStream());
			in = new ObjectInputStream(socket.getInputStream());
		} catch (IOException e) {
			System.out.println("[Migration Service] Could not connect to target platform: " + host + ":"+port);
			return false;
		}
		
		try {
			out.writeObject(snapshot);
			boolean result = in.readBoolean();
			if (result) {
				ams.terminateAgent(agent.getName());
			}
			out.writeObject("DONE");
		} catch (Exception e) {
			System.out.println("[Migration Service] Failed to transfer agent to target platform " + host + ":"+port + ": " + e.getMessage());
			e.printStackTrace();
			
			return false;
		}
			
		try {
			in.close();
			out.close();
			socket.close();
		} catch (Exception e) {
			return false;
		}
		
		return true;
	}
	
	public void startAgent(String name) {
		ams.resumeAgent(name);
	}
	
	public boolean receiveAgent(IMigrationSnapshot snapshot) {
		try {
			ams.createAgent(snapshot.getName(), snapshot);
		} catch (Exception e) {
			System.out.println("[Migration Service] Migration Failure: An agent already exists with name: " + snapshot.getName());
			e.printStackTrace();
			return false;
		}
		
		return true;
	}
	
    @Override
    public void onStart() {
		ams = (AgentManagementService) manager.getServiceByName(AgentManagementService.NAME);
        try {
            port = Integer.parseInt( configuration.getProperty( "port" ) );
            server = new MigrationServer( port, this );
            thread = new Thread( server, "HTTP-MTS" );
            thread.start();
            System.out.println( "Migration Service started on port: " + port );
        }
        catch ( NumberFormatException e ) {
            System.err.println( "HTTP MTS Failed to Start: Port was not configured correctly" );
        }
    }

}
