package com.agentfactory.teleoreactive.interpreter;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.clf.interpreter.Action;
import com.agentfactory.clf.interpreter.CoreUtilities;
import com.agentfactory.clf.interpreter.Module;
import com.agentfactory.clf.interpreter.Sensor;
import com.agentfactory.clf.lang.IFormula;
import com.agentfactory.clf.lang.IMPLIES;
import com.agentfactory.clf.lang.Predicate;
import com.agentfactory.clf.reasoner.QueryResult;
import com.agentfactory.platform.core.AgentCreationException;
import com.agentfactory.teleoreactive.compiler.CodeGeneratorVisitor;
import com.agentfactory.teleoreactive.compiler.IncludesExpanderStream;
import com.agentfactory.teleoreactive.compiler.SimpleNode;
import com.agentfactory.teleoreactive.compiler.TeleoReactiveParser;
import com.agentfactory.teleoreactive.interpreter.mentalstate.TRProgram;

public class ScriptBasedTeleoReactiveAgent extends AbstractTeleoReactiveAgent {

	public ScriptBasedTeleoReactiveAgent(String name) {
		super(name);
	}

	public void construct(String design) throws AgentCreationException {
		List<CodeGeneratorVisitor> agents = new LinkedList<CodeGeneratorVisitor>();
		CodeGeneratorVisitor cgv = new CodeGeneratorVisitor();
		String[] arr = design.split("[\\./]");
		String name = arr[arr.length - 2];
		try {
			TeleoReactiveParser parser = new TeleoReactiveParser(
					IncludesExpanderStream.class.getResourceAsStream("/"
							+ design));
			SimpleNode n = parser.Start();
			cgv.visit(n, null);
			agents.add(cgv);
			if (!name.equals(cgv.getName()))
				throw new AgentCreationException("File " + design
						+ " does not match name given in file... "
						+ cgv.getName());

			if (!parser.errors.isEmpty()) {
				throw new RuntimeException(
						"ERROR: Teleo-Reactive program does not compile: "
								+ design);
			}
			while (cgv.hasSuper()) {
				TeleoReactiveParser parser2 = new TeleoReactiveParser(
						IncludesExpanderStream.class.getResourceAsStream("/"
								+ cgv.getSuper().replace('.', '/') + ".aftr"));

				cgv = new CodeGeneratorVisitor();
				n = parser2.Start();
				cgv.visit(n, null);
				name = cgv.getName();
				if (!name.equals(cgv.getName()))
					throw new AgentCreationException("File " + design
							+ " does not match name given in file... " + name);

				agents.add(0, cgv);
			}
			if (!agents.get(0).getName().equals("AbstractAFTRAgent")) {
				TeleoReactiveParser parser2 = new TeleoReactiveParser(
						IncludesExpanderStream.class
								.getResourceAsStream("/com/agentfactory/teleoreactive/compiler/AbstractAFTRAgent.aftr"));
				cgv = new CodeGeneratorVisitor();
				n = parser2.Start();
				cgv.visit(n, null);
				agents.add(0, cgv);
			}

			for (CodeGeneratorVisitor c : agents) {
				getInfo(c);
			}
			checkFunctions();

		} catch (Exception e) {
			e.printStackTrace();
			throw new AgentCreationException(e.getMessage());
		}
	}

	private void checkFunctions() throws AgentCreationException {
		for (TRProgram t : getPrograms().values()) {
			if (t.isAbstract())
				throw new AgentCreationException("Function " + t.getName()
						+ " must be implemented");
		}

	}

	private void getInfo(CodeGeneratorVisitor cgv) throws Exception {
		for (Predicate id : cgv.actions.keySet()) {
			Action action = (Action) Class.forName(cgv.actions.get(id))
					.newInstance();
			action.setId(id);
			action.setAgent(this);
			addAction(CoreUtilities.presenter.toString(id), action);
		}

		for (Predicate belief : cgv.beliefs) {
			beliefs.addNewInformedBelief(belief);

		}

		for (Predicate id : cgv.sensors.keySet()) {
			Sensor sensor = (Sensor) Class.forName(cgv.sensors.get(id))
					.newInstance();
			sensor.setId(id);
			sensor.setAgent(this);
			addSensor(CoreUtilities.presenter.toString(id), sensor);
		}
		for (Predicate id : cgv.modules.keySet()) {
			Module module = (Module) Class.forName(cgv.modules.get(id))
					.newInstance();
			module.setId(id);
			module.setAgent(this);
			addModule(id, module);
		}

		for (String key : cgv.programs.keySet()) {
			TRProgram p = getPrograms().get(key);
			if (p != null) {
				TRProgram t = cgv.programs.get(key);
				if (p.isAbstract()) {
					if (t.isImplements()) {
//						System.out.println("Implementing function " + key);
						getPrograms().put(key, t);
					} else {
						System.err.println("Function " + key
								+ " should be declared #implements");
					}
				} else {
					if (t.isOverride()) {
						System.out.println("Overriding method: " + key);
						getPrograms().put(key, t);
					} else {
						System.err.println("Function " + key + " in "
								+ cgv.name + " should be defined as #override");
						getPrograms().put(key, t);
					}

				}
			} else {
				getPrograms().put(key, cgv.programs.get(key));
			}
		}

		for (IMPLIES i : cgv.inferanceRules) {
			beliefs.addRule(i);
		}
		for (IMPLIES i : cgv.resolutionRules) {
			BCrules.addRule(i);
		}
		for (IMPLIES i : cgv.truthManRules) {
			beliefs.addContradictionRule(i);
		}

	}

	public QueryResult queryAll(IFormula formula) {
		return null;
	}

	public void removeBelief(Predicate belief) {
	}

}
