/*
 * PlatformGUI.java
 *
 * Created on 14 November 2005, 06:39
 */
package com.agentfactory.visualiser;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyVetoException;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.core.IAgentPlatform;
import com.agentfactory.platform.core.IPlatformService;
import com.agentfactory.platform.core.IPlatformServiceManager;
import com.agentfactory.platform.core.IScheduler;
import com.agentfactory.platform.core.NoSuchAgentException;
import com.agentfactory.service.ams.AgentManagementService;
import com.agentfactory.visualiser.core.IAgentInspector;
import com.agentfactory.visualiser.core.IInspectorFactory;
import com.agentfactory.visualiser.core.IServiceInspectorFactory;
import com.agentfactory.visualiser.core.IStateManager;
import com.agentfactory.visualiser.core.IStateManagerFactory;
import com.agentfactory.visualiser.impl.DefaultInspectorFactory;
import com.agentfactory.visualiser.impl.MessageStore;
import com.agentfactory.visualiser.impl.ServiceInspector;
import com.agentfactory.visualiser.impl.SnapshotManager;

/**
 * 
 * @author Administrator
 */
public class Debugger extends javax.swing.JFrame implements Observer {
   private static final long serialVersionUID = 1L;
   private static final String LOGNAME = Debugger.class.getName();

   private SnapshotManager manager;
   private Map<String, IAgentInspector> inspectorTable;
   private Map<String, ServiceInspector> serviceInspectors;
   private IScheduler scheduler;
   private IAgentPlatform platform;
   private DefaultMutableTreeNode agentsNode;
   private MessageStore messageStore;
   private DefaultMutableTreeNode servicesNode;

   private List<IInspectorFactory> inspectorFactories;
   private List<IServiceInspectorFactory> serviceInspectorFactories;

   IServiceInspectorFactory defaultServiceInspectorFactory =
         new IServiceInspectorFactory() {
            @Override
            public boolean handles( Class<? extends IPlatformService> class1 ) {
               return true;
            }

            @Override
            public ServiceInspector create( IPlatformService service ) {
               ServiceInspector toReturn = new ServiceInspector( service );
               return toReturn;
            }
         };

   static {
      Logger.getLogger( LOGNAME ).setLevel( Level.SEVERE );
   }

   public void start() {
      setSize( 1024, 768 );
      setVisible( true );
   }

   public void setMaxHistory( int maxHistory ) {
      manager.setMaxHistory( maxHistory );
   }

   public void registerInspectorFactory( IInspectorFactory factory ) {
      inspectorFactories.add( factory );
   }

   public void registerServiceInspectorFactory( IServiceInspectorFactory factory ) {
      serviceInspectorFactories.add( factory );
   }

   public void registerStateManagerFactory( IStateManagerFactory factory ) {
      manager.registerStateManagerFactory( factory );
   }

   /** Creates new form PlatformGUI */
   public Debugger( IAgentPlatform platform ) {
      // try setting native look and feel
      try {
         UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
      }
      catch ( Exception e ) {
         System.out.println( "Error setting native look and feel: " + e );
      }

      this.platform = platform;

      // set up the inspector table
      inspectorFactories = new LinkedList<IInspectorFactory>();
      inspectorTable = new HashMap<String, IAgentInspector>();
      serviceInspectors = new HashMap<String, ServiceInspector>();
      serviceInspectorFactories = new LinkedList<IServiceInspectorFactory>();

      // The snapshot manager should be configured to observe events that
      // occur in the agent container.
      manager = new SnapshotManager();
      ( (Observable) platform.getAgentContainer() ).addObserver( manager );

      IPlatformServiceManager psm = platform.getPlatformServiceManager();
      if ( Observable.class.isInstance( psm ) ) {
         ( (Observable) psm ).addObserver( this );
      }

      AgentManagementService ams =
            (AgentManagementService) platform.getPlatformServiceManager().getServiceByName( AgentManagementService.NAME );
      ams.addObserver( this );

      // The platform viewer also observes events that occur in the agent
      // container to ensure the update of the agent model.
      scheduler = platform.getScheduler();
      messageStore = new MessageStore();

      initComponents();

      setTitle( "Debugger opened on: " + platform.getFullName() );

      // TODO: fix: node.init(platform, ams);

      platformState.setRowHeight( 20 );
      platformState.setCellRenderer( new DefaultTreeCellRenderer() {
         private static final long serialVersionUID = 1L;
         ImageIcon[] state;
         // ImageIcon agents;

         { // Initialization block...
            state = new ImageIcon[ 4 ];
            state[ 0 ] =
                  new ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/state_suspended.gif" ) );
            state[ 1 ] =
                  new ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/state_active.gif" ) );
            state[ 2 ] =
                  new ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/state_waiting.gif" ) );
            state[ 3 ] =
                  new ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/state_suspended.gif" ) );
            // agents = new
            // ImageIcon(getClass().getResource("/com/agentfactory/visualiser/image/state_suspended.gif"));
         }

         @Override
         public java.awt.Component getTreeCellRendererComponent( JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean focused ) {
            JLabel label =
                  (JLabel) super.getTreeCellRendererComponent( tree, value.toString(), selected, expanded, leaf, row, focused );
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
            if ( IAgent.class.isInstance( node.getUserObject() ) ) {
               IAgent agent = ( (IAgent) node.getUserObject() );
               label.setText( agent.getName() );
               if ( agent.getState() < 4 ) {
                  label.setIcon( state[ agent.getState() ] );
               }
            }
            else if ( IPlatformService.class.isInstance( node.getUserObject() ) ) {
               IPlatformService service =
                     (IPlatformService) node.getUserObject();
               label.setText( service.getName() );
            }

            return label;
         }
      } );

      platformState.addMouseListener( new MouseListener() {
         public void mouseClicked( MouseEvent e ) {
            if ( e.getClickCount() == 2 ) {
               TreePath treePath =
                     platformState.getPathForLocation( e.getX(), e.getY() );
               if ( treePath != null ) {
                  TreeNode node = (TreeNode) treePath.getLastPathComponent();
                  DefaultMutableTreeNode dnode = (DefaultMutableTreeNode) node;
                  if ( IAgent.class.isInstance( dnode.getUserObject() ) ) {
                     IAgent agent = ( (IAgent) dnode.getUserObject() );
                     openInspector( agent );
                  }
                  else if ( IPlatformService.class.isInstance( dnode.getUserObject() ) ) {
                     IPlatformService service =
                           (IPlatformService) dnode.getUserObject();
                     openServiceInspector( service );
                  }
               }
            }
         }

         public void mouseReleased( MouseEvent mouseEvent ) {
            if ( mouseEvent.isPopupTrigger() ) {
               showPopup( mouseEvent );
            }
         }

         public void mousePressed( MouseEvent mouseEvent ) {
            if ( mouseEvent.isPopupTrigger() ) {
               showPopup( mouseEvent );
            }
         }

         public void mouseExited( java.awt.event.MouseEvent mouseEvent ) {}

         public void mouseEntered( java.awt.event.MouseEvent mouseEvent ) {}
      } );
   }

   private void initComponents() {

      try {
         BufferedImage image =
               ImageIO.read( getClass().getResource( "/com/agentfactory/visualiser/image/af-icon.png" ) );
         this.setIconImage( image );
      }
      catch ( IOException e ) {
         e.printStackTrace();
      }

      jScrollPane1 = new javax.swing.JScrollPane();
      accessTree = new javax.swing.JTree();
      jPanel1 = new javax.swing.JPanel();
      jToolBar1 = new javax.swing.JToolBar();
      stopAll = new javax.swing.JButton();
      startAll = new javax.swing.JButton();
      jSeparator1 = new javax.swing.JToolBar.Separator();
      auml = new javax.swing.JButton();
      jSplitPane1 = new javax.swing.JSplitPane();
      desktop = new javax.swing.JDesktopPane();
      jScrollPane3 = new javax.swing.JScrollPane();
      jMenuBar1 = new javax.swing.JMenuBar();
      jMenu1 = new javax.swing.JMenu();
      jMenuItem2 = new javax.swing.JMenuItem();
      jMenu2 = new javax.swing.JMenu();
      jMenuItem1 = new javax.swing.JMenuItem();
      jMenu3 = new javax.swing.JMenu();

      jScrollPane1.setViewportView( accessTree );

      setDefaultCloseOperation( javax.swing.WindowConstants.EXIT_ON_CLOSE );

      setTitle( "Agent Factory Debugger V1.0.0" );

      jPanel1.setLayout( new java.awt.FlowLayout( java.awt.FlowLayout.LEFT ) );

      stopAll.setIcon( new javax.swing.ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/pauseall.png" ) ) );
      stopAll.setFocusable( false );
      stopAll.setText( "Pause" );
      stopAll.setHorizontalTextPosition( javax.swing.SwingConstants.RIGHT );
      // stopAll.setMaximumSize(new java.awt.Dimension(100, 50));
      // stopAll.setMinimumSize(new java.awt.Dimension(100, 50));
      // stopAll.setPreferredSize(new java.awt.Dimension(100, 50));
      stopAll.setVerticalTextPosition( javax.swing.SwingConstants.CENTER );
      Font font = stopAll.getFont().deriveFont( 10 );
      stopAll.setFont( font );
      stopAll.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed( java.awt.event.ActionEvent evt ) {
            if ( agentsNode != null ) {
               Enumeration<?> e = agentsNode.children();
               while ( e.hasMoreElements() ) {
                  DefaultMutableTreeNode node =
                        (DefaultMutableTreeNode) e.nextElement();
                  if ( IAgent.class.isInstance( node.getUserObject() ) ) {
                     IAgent agent = ( (IAgent) node.getUserObject() );
                     scheduler.suspend( agent.getName() );
                  }
               }
            }
         }
      } );
      jToolBar1.add( stopAll );

      startAll.setIcon( new javax.swing.ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/startall.png" ) ) );
      startAll.setFocusable( false );
      startAll.setHorizontalTextPosition( javax.swing.SwingConstants.RIGHT );
      // startAll.setMaximumSize(new java.awt.Dimension(100, 50));
      // startAll.setMinimumSize(new java.awt.Dimension(100, 50));
      // startAll.setPreferredSize(new java.awt.Dimension(100, 50));
      startAll.setVerticalTextPosition( javax.swing.SwingConstants.CENTER );
      startAll.setText( "Start" );
      startAll.setFont( font );
      startAll.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed( java.awt.event.ActionEvent evt ) {
            if ( agentsNode != null ) {
               Enumeration<?> e = agentsNode.children();
               while ( e.hasMoreElements() ) {
                  DefaultMutableTreeNode node =
                        (DefaultMutableTreeNode) e.nextElement();
                  if ( IAgent.class.isInstance( node.getUserObject() ) ) {
                     IAgent agent = ( (IAgent) node.getUserObject() );
                     scheduler.resume( agent.getName() );
                  }
               }
            }
         }
      } );
      jToolBar1.add( startAll );
      jToolBar1.add( jSeparator1 );

      auml.setIcon( new javax.swing.ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/auml.gif" ) ) ); // NOI18N
      auml.setFocusable( false );
      auml.setText( "Agent UML" );
      auml.setFont( font );
      auml.setHorizontalTextPosition( javax.swing.SwingConstants.RIGHT );
      // auml.setMaximumSize(new java.awt.Dimension(100, 50));
      // auml.setMinimumSize(new java.awt.Dimension(100, 50));
      // auml.setPreferredSize(new java.awt.Dimension(100, 50));
      auml.setVerticalTextPosition( javax.swing.SwingConstants.CENTER );
      auml.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed( java.awt.event.ActionEvent evt ) {
            if ( aumlView == null ) {
               aumlView = new ProtocolFrame( messageStore );
               desktop.add( aumlView );
               messageStore.addObserver( aumlView );
            }
            else {
               aumlView.moveToFront();
            }
            aumlView.setVisible( true );
         }
      } );
      jToolBar1.add( auml );

      jPanel1.add( jToolBar1 );

      getContentPane().add( jPanel1, java.awt.BorderLayout.NORTH );

      jSplitPane1.setRightComponent( desktop );

      DefaultMutableTreeNode platformNode =
            new DefaultMutableTreeNode( "Platform Details" );
      platformNode.add( new DefaultMutableTreeNode( "Name: "
            + platform.getName() ) );
      platformNode.add( new DefaultMutableTreeNode( "Domain: "
            + platform.getDomain() ) );
      agentsNode = new DefaultMutableTreeNode( "Agents" );
      platformNode.add( agentsNode );
      servicesNode = new DefaultMutableTreeNode( "Platform Services" );
      for ( IPlatformService service : platform.getPlatformServiceManager().getFilteredServices( IPlatformService.class ) ) {
         DefaultMutableTreeNode node = new DefaultMutableTreeNode( service );
         servicesNode.add( node );
         serviceNodeMap.put( service, node );
      }
      platformNode.add( servicesNode );

      platformState = new javax.swing.JTree( platformNode );

      jScrollPane3.setViewportView( platformState );

      jSplitPane1.setLeftComponent( jScrollPane3 );
      jSplitPane1.setDividerLocation( 220 );

      getContentPane().add( jSplitPane1, java.awt.BorderLayout.CENTER );

      jMenu1.setText( "File" );

      jMenuItem2.setMnemonic( 'q' );
      jMenuItem2.setAccelerator( KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_MASK));
      jMenuItem2.setText( "Quit" );
      jMenuItem2.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed( java.awt.event.ActionEvent evt ) {
            System.exit( 0 );
         }
      } );
      jMenu1.add( jMenuItem2 );

      jMenuBar1.add( jMenu1 );

      jMenu2.setMnemonic( 'T' );
      jMenu2.setText( "Tools" );

      jMenuItem1.setMnemonic( 'S' );
      jMenuItem1.setText( "Message Sender" );
      jMenuItem1.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed( java.awt.event.ActionEvent evt ) {}
      } );
      jMenu2.add( jMenuItem1 );

      jMenuBar1.add( jMenu2 );

      jMenu3.setText( "Views" );
      jMenuBar1.add( jMenu3 );

      setJMenuBar( jMenuBar1 );

      pack();
   }

   private ProtocolFrame aumlView;

   // Variables declaration - do not modify//GEN-BEGIN:variables
   private javax.swing.JTree accessTree;
   private javax.swing.JButton auml;
   private javax.swing.JDesktopPane desktop;
   private javax.swing.JMenu jMenu1;
   private javax.swing.JMenu jMenu2;
   private javax.swing.JMenu jMenu3;
   private javax.swing.JMenuBar jMenuBar1;
   private javax.swing.JMenuItem jMenuItem1;
   private javax.swing.JMenuItem jMenuItem2;
   private javax.swing.JPanel jPanel1;
   private javax.swing.JScrollPane jScrollPane1;
   private javax.swing.JScrollPane jScrollPane3;
   private javax.swing.JToolBar.Separator jSeparator1;
   private javax.swing.JSplitPane jSplitPane1;
   private javax.swing.JToolBar jToolBar1;
   private javax.swing.JTree platformState;
   private javax.swing.JButton startAll;
   private javax.swing.JButton stopAll;

   // End of variables declaration//GEN-END:variables

   private void showPopup( MouseEvent mouseEvent ) {
      TreePath path = platformState.getSelectionPath();
      DefaultMutableTreeNode node =
            (DefaultMutableTreeNode) path.getLastPathComponent();
      if ( ( path != null )
            && ( IAgent.class.isInstance( node.getUserObject() ) ) ) {
         JPopupMenu menu = new JPopupMenu();
         JMenuItem item = new JMenuItem( "Open Inspector" );
         item.addActionListener( new ActionListener() {

            public void actionPerformed( ActionEvent e ) {
               TreePath path = platformState.getSelectionPath();
               DefaultMutableTreeNode node =
                     (DefaultMutableTreeNode) path.getLastPathComponent();
               IAgent agent = ( (IAgent) node.getUserObject() );
               openInspector( agent );
            }
         } );
         item.setFont( item.getFont().deriveFont( Font.BOLD ) );
         menu.add( item );
         menu.add( new JSeparator() );

         item = new JMenuItem( "Start" );
         item.setIcon( new ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/start.gif" ) ) );
         item.addActionListener( new ActionListener() {

            public void actionPerformed( ActionEvent e ) {
               TreePath[] values = platformState.getSelectionPaths();
               if ( values != null ) {
                  for ( TreePath path : values ) {
                     DefaultMutableTreeNode node =
                           (DefaultMutableTreeNode) path.getLastPathComponent();
                     if ( IAgent.class.isInstance( node.getUserObject() ) ) {
                        IAgent agent = ( (IAgent) node.getUserObject() );
                        scheduler.resume( agent.getName() );
                     }
                  }
               }
            }
         } );
         menu.add( item );

         item = new JMenuItem( "Stop" );
         item.setIcon( new ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/stop.gif" ) ) );
         item.addActionListener( new ActionListener() {

            public void actionPerformed( ActionEvent e ) {
               TreePath[] values = platformState.getSelectionPaths();
               if ( values != null ) {
                  for ( TreePath path : values ) {
                     DefaultMutableTreeNode node =
                           (DefaultMutableTreeNode) path.getLastPathComponent();
                     if ( IAgent.class.isInstance( node.getUserObject() ) ) {
                        IAgent agent = ( (IAgent) node.getUserObject() );
                        scheduler.suspend( agent.getName() );
                     }
                  }
               }
            }
         } );
         menu.add( item );

         item = new JMenuItem( "Step" );
         item.setIcon( new ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/step-forward.png" ) ) );
         item.addActionListener( new ActionListener() {

            public void actionPerformed( ActionEvent e ) {
               TreePath[] values = platformState.getSelectionPaths();
               if ( values != null ) {
                  for ( TreePath path : values ) {
                     DefaultMutableTreeNode node =
                           (DefaultMutableTreeNode) path.getLastPathComponent();
                     if ( IAgent.class.isInstance( node.getUserObject() ) ) {
                        IAgent agent = ( (IAgent) node.getUserObject() );
                        scheduler.step( agent.getName() );
                     }
                  }
               }
            }
         } );
         menu.add( item );

         menu.add( new JSeparator() );
         item = new JMenuItem( "Terminate" );
         item.setIcon( new ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/remove.gif" ) ) );
         item.addActionListener( new ActionListener() {

            public void actionPerformed( ActionEvent e ) {
               TreePath[] values = platformState.getSelectionPaths();
               if ( values != null ) {
                  for ( TreePath path : values ) {
                     DefaultMutableTreeNode node =
                           (DefaultMutableTreeNode) path.getLastPathComponent();
                     if ( IAgent.class.isInstance( node.getUserObject() ) ) {
                        IAgent agent = ( (IAgent) node.getUserObject() );
                        System.out.println( "Remove: " + agent.getName() );
                     }
                  }
               }
            }
         } );
         menu.add( item );

         menu.show( this, mouseEvent.getX(), mouseEvent.getY() );
      }
   }

   private IInspectorFactory getInspectorFactory( IStateManager manager ) {
      for ( IInspectorFactory factory : inspectorFactories ) {
         if ( factory.handles( manager ) )
            return factory;
      }

      return new DefaultInspectorFactory();
   }

   private void openInspector( IAgent agent ) {
      IAgentInspector inspector = inspectorTable.get( agent.getName() );
      if ( inspector == null ) {
         IStateManager stateManager = manager.getStateManager( agent.getName() );
         if ( stateManager == null ) {
            System.out.println( "WARNING: Could not Open Inspector: NO STATE MANAGER" );
            return;
         }
         inspector =
               getInspectorFactory( stateManager ).create( stateManager, scheduler );
         inspectorTable.put( agent.getName(), inspector );
         desktop.add( inspector.getInternalFrame() );

         // maximise inspector by default
         try {
            inspector.getInternalFrame().setMaximum( true );
         }
         catch ( PropertyVetoException e ) {
            // failed to maximise inspector
         }
      }
      else {
         inspector.getInternalFrame().moveToFront();
      }
      inspector.getInternalFrame().setVisible( true );
   }

   private IServiceInspectorFactory getServiceInspectorFactory( IPlatformService service ) {
      for ( IServiceInspectorFactory factory : serviceInspectorFactories ) {
         if ( factory.handles( service.getClass() ) )
            return factory;
      }
      return defaultServiceInspectorFactory;
   }

   private void openServiceInspector( IPlatformService service ) {
      ServiceInspector inspector = serviceInspectors.get( service.getName() );

      if ( inspector == null ) {
         inspector = getServiceInspectorFactory( service ).create( service );
         serviceInspectors.put( service.getName(), inspector );
         desktop.add( inspector.getInternalFrame() );
         // maximise inspector by default
         try {
            inspector.getInternalFrame().setMaximum( true );
         }
         catch ( PropertyVetoException e ) {
            // failed to maximise inspector
         }
      }
      else {
         inspector.getInternalFrame().moveToFront();
      }

      inspector.getInternalFrame().setVisible( true );
   }

   private void updateTreeView() {
      SwingUtilities.invokeLater( new Runnable() {
         public void run() {
            int i = 0;
            while ( i < platformState.getRowCount() ) {
               platformState.expandRow( i++ );
            }
            platformState.updateUI();
         }
      } );
   }

   private Map<IAgent, DefaultMutableTreeNode> agentNodeMap =
         new HashMap<IAgent, DefaultMutableTreeNode>();
   private Map<IPlatformService, DefaultMutableTreeNode> serviceNodeMap =
         new HashMap<IPlatformService, DefaultMutableTreeNode>();

   public synchronized void update( Observable o, Object arg ) {
      if ( o instanceof AgentManagementService && arg instanceof String ) {
         String[] info = ( (String) arg ).split( " " );
         if ( info[ 0 ].equals( "created" ) ) {
            try {
               IAgent agent =
                     platform.getAgentContainer().getAgentByName( info[ 1 ] );
               DefaultMutableTreeNode node = new DefaultMutableTreeNode( agent );
               agentsNode.add( node );
               agentNodeMap.put( agent, node );
               if ( Observable.class.isInstance( agent ) ) {
                  ( (Observable) agent ).addObserver( messageStore );
               }
            }
            catch ( NoSuchAgentException ex ) {
               Logger.getLogger( Debugger.class.getName() ).log( Level.SEVERE, null, ex );
            }
            updateTreeView();
         }
         else if ( info[ 0 ].equals( "terminated" ) ) {
            try {
               IAgent agent =
                     platform.getAgentContainer().getAgentByName( info[ 1 ] );
               agentsNode.remove( agentNodeMap.remove( agent ) );
               if ( Observable.class.isInstance( agent ) ) {
                  ( (Observable) agent ).deleteObserver( messageStore );
               }
               IAgentInspector inspector = inspectorTable.remove( info[ 1 ] );
               if ( inspector != null ) {
                  System.out.println( "found inspector" );
                  inspector.getInternalFrame().doDefaultCloseAction();
               }
            }
            catch ( NoSuchAgentException ex ) {
               Logger.getLogger( Debugger.class.getName() ).log( Level.SEVERE, null, ex );
            }
            updateTreeView();
         }
         else if ( info[ 0 ].equals( "suspended" )
               || info[ 0 ].equals( "resumed" ) ) {
            updateTreeView();
         }
      }
      if ( IPlatformServiceManager.class.isInstance( o ) ) {
         IPlatformServiceManager psm = (IPlatformServiceManager) o;
         String[] info = ( (String) arg ).split( " " );
         if ( info[ 0 ].equals( "registered" ) ) {
            IPlatformService service = psm.getServiceByName( info[ 1 ] );
            DefaultMutableTreeNode node = new DefaultMutableTreeNode( service );
            servicesNode.add( node );
            serviceNodeMap.put( service, node );
            updateTreeView();
         }
         else if ( info[ 0 ].equals( "unregistered" ) ) {
            IPlatformService service = psm.getServiceByName( info[ 1 ] );
            DefaultMutableTreeNode node = serviceNodeMap.remove( service );
            servicesNode.remove( node );
            updateTreeView();
         }
      }
   }
}
