package com.agentfactory.visualiser;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.platform.impl.DefaultRunConfiguration;
import com.agentfactory.visualiser.core.IInspectorFactory;
import com.agentfactory.visualiser.core.IServiceInspectorFactory;
import com.agentfactory.visualiser.core.IStateManagerFactory;

public class DebuggerRunConfiguration extends DefaultRunConfiguration {
	private List<IInspectorFactory> agentInspectors = new LinkedList<IInspectorFactory>();
	private List<IStateManagerFactory> stateManagers = new LinkedList<IStateManagerFactory>();
	private List<IServiceInspectorFactory> serviceInspectors = new LinkedList<IServiceInspectorFactory>();
	
	@Override
	public void configure() {
		super.configure();

		Debugger debugger = new Debugger(this.getAgentPlatform());
        for (IInspectorFactory factory : agentInspectors) {
        	debugger.registerInspectorFactory(factory);
        }
        for (IStateManagerFactory factory : stateManagers) {
        	debugger.registerStateManagerFactory(factory);
        }
        for (IServiceInspectorFactory factory : serviceInspectors) {
        	debugger.registerServiceInspectorFactory(factory);
        }
        
        debugger.start();
	}
	
	protected void addLanguageSupport(IInspectorFactory iFactory, IStateManagerFactory sFactory) {
		agentInspectors.add(iFactory);
		stateManagers.add(sFactory);
	}
	
	protected void addServiceInspectorFactory(IServiceInspectorFactory factory) {
		serviceInspectors.add(factory);
	}
}
