package com.agentfactory.visualiser;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.TitledBorder;

import com.agentfactory.platform.core.AgentID;


/**
 * A filter panel contains a list of agent IDs, user can multi-select
 * in the list then only visualise interactions between selected agents.
 * 
 * @author Zhenhua Yang
 *
 */
public class FilterPanel extends JPanel {
	
   private static final long serialVersionUID = -7741749349668974152L;
   private Set<AgentID> allAIDs;
	private JList listbox;
	private JCheckBox enableFilter;
	private JButton filterButton;
	
	/**
	 * Constructor with the full list of agent IDs
	 * 
	 * @param allAIDSet - full list of agent IDs
	 */
	public FilterPanel(Set<AgentID> allAIDSet) {
		this.allAIDs = allAIDSet;
		buildUI();
	}

	private void buildUI() {
		this.setLayout(new BorderLayout());
		
		enableFilter = new JCheckBox("Enable Agent ID filter");
		enableFilter.setPreferredSize(new Dimension(160, 40));
		TitledBorder filterBorder = BorderFactory.createTitledBorder("Agent ID Filter: ");
		this.setBorder(filterBorder);
		
		add(enableFilter, BorderLayout.NORTH);
      
		listbox = new JList();
		listbox.setListData(allAIDs.toArray());

		JScrollPane scroll = new JScrollPane(listbox);
		add(scroll, BorderLayout.CENTER);
		
		filterButton = new JButton("Update Filter");
		add(filterButton, BorderLayout.SOUTH);
	}
	
	/**
	 * Entry point for user-defined event handling
	 *  
	 * @param e - event handler
	 */
	public void setEventHandler(ActionListener e) {
		filterButton.addActionListener(e);
	}
	
	/**
	 * Gets the list of agents selected.
	 * 
	 * @return a subset of agent IDs
	 */
	public Set<AgentID> getSelectedAIDs() {
		if (!enableFilter.isSelected()) return allAIDs;
		
		TreeSet<AgentID> aidSet = new TreeSet<AgentID>();
		for (Object o: listbox.getSelectedValues()) {
			AgentID aid = (AgentID) o;
			aidSet.add(aid);
		}
		return aidSet;
	}
}