package com.agentfactory.visualiser;


import java.util.Date;
import java.util.List;

import com.agentfactory.platform.core.AgentID;
import com.agentfactory.platform.core.IMessage;



/**
 * A InteractionRecord = a StringMessage + LogTime, and some helper methods.
 * 
 * @author Zhenhua Yang
 */
public class InteractionRecord implements Comparable<InteractionRecord> {

	private IMessage message;
	private Date logTime;
	
	public InteractionRecord(Date date, IMessage msg) {
		message = msg;
		logTime = date;
	}

	public IMessage getStringMessage() {
		return message;
	}
	
	public AgentID getSenderSimpleAID() {
		return message.getSender();
	}
	
	/**
	 * Convert to simple AID format
	 * 
	 * @return
	 */
	public List<AgentID> getReceiverSimpleAIDs() {
		return message.getReceivers();
	}

	public Date getLogTime() {
		return logTime;
	}
	
	/**
	 * used to sort a list of interaction records: by log time, ascending order
	 */
	public int compareTo(InteractionRecord o) {
		return logTime.compareTo(o.getLogTime());
	}
	
	public String toString() {
		return message.getPerformative() + ": [" + getReceiverSimpleAIDs().get(0) + "] MESSAGE: [" + message.getContent() + "] @" + MiscUtils.formatSentTime(logTime);
	}
}
