/*
 * NewJInternalFrame.java
 *
 * Created on November 12, 2007, 12:26 PM
 */

package com.agentfactory.visualiser.impl;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;

import com.agentfactory.platform.core.IScheduler;
import com.agentfactory.visualiser.core.IAgentInspector;
import com.agentfactory.visualiser.core.ISnapshot;
import com.agentfactory.visualiser.core.IStateManager;

/**
 * 
 * @author remcollier
 */
public class DefaultAgentInspector extends javax.swing.JInternalFrame implements IAgentInspector, Observer {
   private static final long serialVersionUID = 1L;
   private IScheduler scheduler;
   private IStateManager manager;
   private ISnapshot snapshot;

   /** Creates new form NewJInternalFrame */
   public DefaultAgentInspector() {
      initComponents();

      addTabs( jTabbedPane1 );
   }

   public void addTabs( JTabbedPane jTabbedPane ) {
      jTabbedPane.addTab( "Services", new ServicesPanel() );
      jTabbedPane.addTab( "Messages", new MessagesPanel() );
   }

   public void init( IStateManager manager, IScheduler scheduler ) {
      this.setTitle( "Agent Inspector on: " + manager.getAgentName() );
      this.manager = manager;
      this.scheduler = scheduler;
      snapshot = manager.getCurrentSnapshot();
      updateView();
   }

   public void update( Observable source, Object event ) {
      if ( event.equals( IStateManager.NEW_SNAPSHOT ) ) {
         snapshot = manager.getCurrentSnapshot();
         updateView();
      }
   }

   private void updateView() {
      iteration.setText( "" + snapshot.numIterations() );

      for ( int i = 0; i < jTabbedPane1.getTabCount(); i++ ) {
         if ( jTabbedPane1.getComponentAt( i ) instanceof InspectorPanel ) {
            ( (InspectorPanel) jTabbedPane1.getComponentAt( i ) ).update( snapshot );
         }
      }
   }

   private void initComponents() {

      jPanel1 = new javax.swing.JPanel();
      jPanel2 = new javax.swing.JPanel();
      jLabel1 = new javax.swing.JLabel();
      iteration = new javax.swing.JTextField();
      jPanel3 = new javax.swing.JPanel();
      stop = new javax.swing.JButton();
      start = new javax.swing.JButton();
      step = new javax.swing.JButton();
      jSeparator2 = new javax.swing.JSeparator();
      first = new javax.swing.JButton();
      prev = new javax.swing.JButton();
      next = new javax.swing.JButton();
      last = new javax.swing.JButton();
      jTabbedPane1 = new javax.swing.JTabbedPane();

      setClosable( true );
      setDefaultCloseOperation( javax.swing.WindowConstants.HIDE_ON_CLOSE );
      setIconifiable( true );
      setMaximizable( true );
      setResizable( true );
      setTitle( "Default Agent Inspector" );
      setMinimumSize( new java.awt.Dimension( 200, 150 ) );

      jPanel1.setLayout( new java.awt.BorderLayout() );

      jPanel2.setLayout( new java.awt.FlowLayout( java.awt.FlowLayout.RIGHT ) );

      jLabel1.setText( "Iteration:" );
      jPanel2.add( jLabel1 );

      iteration.setColumns( 7 );
      iteration.setHorizontalAlignment( javax.swing.JTextField.RIGHT );
      iteration.setText( "0" );
      jPanel2.add( iteration );

      JButton go = new JButton( "Go" );
      go.addActionListener( new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent arg0 ) {
            int iter = Integer.parseInt( iteration.getText() );
            if ( !manager.setIteration( iter ) ) {
               JOptionPane.showMessageDialog( getInternalFrame(), "This iteration is not available" );
            }
         }
      } );
      jPanel2.add( go );
      jPanel1.add( jPanel2, java.awt.BorderLayout.EAST );

      jPanel3.setLayout( new java.awt.FlowLayout( java.awt.FlowLayout.LEFT, 0, 5 ) );

      stop.setIcon( new javax.swing.ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/pause.png" ) ) ); // NOI18N
      stop.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed( java.awt.event.ActionEvent evt ) {
            scheduler.suspend( manager.getAgentName() );
         }
      } );
      jPanel3.add( stop );

      start.setIcon( new javax.swing.ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/start.png" ) ) ); // NOI18N
      start.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed( java.awt.event.ActionEvent evt ) {
            scheduler.resume( manager.getAgentName() );
         }
      } );
      jPanel3.add( start );

      step.setIcon( new javax.swing.ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/step-forward.png" ) ) ); // NOI18N
      step.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed( java.awt.event.ActionEvent evt ) {
            scheduler.step( manager.getAgentName() );
         }
      } );
      jPanel3.add( step );

      jSeparator2.setOrientation( javax.swing.SwingConstants.VERTICAL );
      jPanel3.add( jSeparator2 );

      first.setIcon( new javax.swing.ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/skip-backward.png" ) ) ); // NOI18N
      first.setMaximumSize( new java.awt.Dimension( 36, 32 ) );
      first.setPreferredSize( new java.awt.Dimension( 36, 32 ) );
      first.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed( java.awt.event.ActionEvent evt ) {
            manager.first();
         }
      } );
      jPanel3.add( first );

      prev.setIcon( new javax.swing.ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/seek-backward.png" ) ) ); // NOI18N
      prev.setMaximumSize( new java.awt.Dimension( 36, 32 ) );
      prev.setPreferredSize( new java.awt.Dimension( 36, 32 ) );
      prev.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed( java.awt.event.ActionEvent evt ) {
            manager.prev();
         }
      } );
      jPanel3.add( prev );

      next.setIcon( new javax.swing.ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/seek-forward.png" ) ) ); // NOI18N
      next.setMaximumSize( new java.awt.Dimension( 36, 32 ) );
      next.setPreferredSize( new java.awt.Dimension( 36, 32 ) );
      next.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed( java.awt.event.ActionEvent evt ) {
            manager.next();
         }
      } );
      jPanel3.add( next );

      last.setIcon( new javax.swing.ImageIcon( getClass().getResource( "/com/agentfactory/visualiser/image/skip-forward.png" ) ) ); // NOI18N
      last.setMaximumSize( new java.awt.Dimension( 36, 32 ) );
      last.setPreferredSize( new java.awt.Dimension( 36, 32 ) );
      last.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed( java.awt.event.ActionEvent evt ) {
            manager.last();
         }
      } );
      jPanel3.add( last );

      jPanel1.add( jPanel3, java.awt.BorderLayout.WEST );

      getContentPane().add( jPanel1, java.awt.BorderLayout.NORTH );
      getContentPane().add( jTabbedPane1, java.awt.BorderLayout.CENTER );

      pack();
   }

   public JInternalFrame getInternalFrame() {
      return this;
   }

   private javax.swing.JButton first;
   private javax.swing.JTextField iteration;
   private javax.swing.JLabel jLabel1;
   private javax.swing.JPanel jPanel1;
   private javax.swing.JPanel jPanel2;
   private javax.swing.JPanel jPanel3;
   private javax.swing.JSeparator jSeparator2;
   private javax.swing.JTabbedPane jTabbedPane1;
   private javax.swing.JButton last;
   private javax.swing.JButton next;
   private javax.swing.JButton prev;
   private javax.swing.JButton start;
   private javax.swing.JButton step;
   private javax.swing.JButton stop;
}
